import asyncio
import mimetypes
import os
import smtplib
import time
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import pyautogui
from bs4 import BeautifulSoup
from playwright.async_api import async_playwright
from rich.console import Console
from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()


def send_email(
    smtp_server: str,
    smtp_port: int,
    smtp_user: str,
    smtp_password: str,
    message_text: str,
    subject: str,
    to: str,
) -> None:
    """
    Envia um e-mail com os dados recebidos.

    Args:
        smtp_server (str): Servidor SMTP
        smtp_port (int): Porta do servidor SMTP
        smtp_user (str): Usuário do servidor SMTP
        smtp_password (str): Senha do usuário do servidor SMTP
        message_text (str): Corpo do e-mail
        subject (str): Assunto do e-mail
        to (str): Destinatário do e-mail
    """
    if not smtp_server:
        raise ValueError("Servidor SMTP não pode ser nulo")
    if not isinstance(smtp_port, int):
        raise ValueError("Porta do servidor SMTP deve ser um inteiro")
    if not smtp_user:
        raise ValueError("Usuário do servidor SMTP não pode ser nulo")
    if not smtp_password:
        raise ValueError("Senha do usuário do servidor SMTP não pode ser nula")
    if not message_text:
        raise ValueError("Corpo do e-mail não pode ser nulo")
    if not subject:
        raise ValueError("Assunto do e-mail não pode ser nulo")
    if not to:
        raise ValueError("Destinatário do e-mail não pode ser nulo")

    message = create_message_with_attachment(smtp_user, to, subject, message_text)
    try:
        send_message(
            smtp_server, smtp_port, smtp_user, smtp_password, smtp_user, to, message
        )
    except Exception as e:
        err_msg = f"Erro ao enviar e-mail: {e}"
        console.print(err_msg, style="bold red")
        logger.error(err_msg)


def create_message_with_attachment(
    sender: str, to: str, subject: str, message_text: str, file: str | None = None
) -> str:
    """
    Cria uma mensagem com um anexo.

    Args:
        sender (str): Remetente do email
        to (str): Destinatário do email
        subject (str): Assunto do email
        message_text (str): Corpo do email
        file (str | None): Caminho do arquivo a ser anexado (opcional)

    Returns:
        str: A mensagem com o anexo no formato de string
    """
    if not sender:
        raise ValueError("Remetente do email não pode ser nulo")
    if not to:
        raise ValueError("Destinatário do email não pode ser nulo")
    if not subject:
        raise ValueError("Assunto do email não pode ser nulo")
    if not message_text:
        raise ValueError("Corpo do email não pode ser nulo")

    message = MIMEMultipart()
    message["to"] = to
    message["from"] = sender
    message["subject"] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    if file:
        if not os.path.exists(file):
            raise FileNotFoundError(f"Arquivo {file} não encontrado")
        if not os.path.isfile(file):
            raise ValueError(f"{file} não é um arquivo")

        content_type, encoding = mimetypes.guess_type(file)

        if content_type is None or encoding is not None:
            content_type = "application/octet-stream"

        main_type, sub_type = content_type.split("/", 1)
        with open(file, "rb") as f:
            attachment = MIMEBase(main_type, sub_type)
            attachment.set_payload(f.read())
            encoders.encode_base64(attachment)
            attachment.add_header(
                "Content-Disposition", "attachment", filename=os.path.basename(file)
            )
            attachment.add_header("Content-Transfer-Encoding", "base64")
            message.attach(attachment)

    return message.as_string()


def send_message(
    smtp_server: str,
    smtp_port: int,
    smtp_user: str,
    smtp_password: str,
    sender: str,
    recipient: str,
    message: str,
) -> None:
    """
    Envia uma mensagem via SMTP.

    Args:
        smtp_server (str): Servidor SMTP
        smtp_port (int): Porta do servidor SMTP
        smtp_user (str): Usuário do servidor SMTP
        smtp_password (str): Senha do usuário do servidor SMTP
        sender (str): Remetente do e-mail
        recipient (str): Destinatário do e-mail
        message (str): Corpo do e-mail

    Returns:
        None
    """
    if not smtp_server or not smtp_port or not smtp_user or not smtp_password:
        raise ValueError("Parâmetros SMTP não podem ser nulos")

    if not sender or not recipient or not message:
        raise ValueError("Parâmetros de e-mail não podem ser nulos")

    try:
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.sendmail(sender, recipient, message)
            logger.info("Mensagem enviada com sucesso")
    except smtplib.SMTPAuthenticationError:
        console.print(f"Erro de autenticação. Verifique seu nome de usuário e senha.\n")
        logger.error("Erro de autenticação. Verifique seu nome de usuário e senha.")
    except smtplib.SMTPConnectError:
        console.print(
            f"Não foi possível conectar ao servidor SMTP. Verifique o endereço e a porta.\n"
        )
        logger.error(
            "Não foi possível conectar ao servidor SMTP. Verifique o endereço e a porta."
        )
    except smtplib.SMTPRecipientsRefused:
        console.print(
            f"O destinatário foi rejeitado. Verifique o endereço do destinatário.\n"
        )
        logger.error(
            "O destinatário foi rejeitado. Verifique o endereço do destinatário."
        )
    except smtplib.SMTPSenderRefused:
        console.print(
            f"O remetente foi rejeitado. Verifique o endereço do remetente..\n"
        )
        logger.error("O remetente foi rejeitado. Verifique o endereço do remetente.")
    except smtplib.SMTPDataError as e:
        console.print(f"Erro ao enviar dados: {e}\n")
        logger.error(f"Erro ao enviar dados: {e}")
    except Exception as error:
        console.print(f"Ocorreu um erro ao enviar a mensagem: {error} \n")
        logger.error("Ocorreu um erro ao enviar a mensagem: %s" % error)


async def ecac_estadual_sc(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que realiza a consulta de caixa postal do ECAC Estadual de SC.

    """
    try:
        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()
        console.print(
            f"Resolução da tela: Width: {screen_width} - Height{screen_height}...\n"
        )
        console.print(f"Task:{task}")

        console.print("Realizando as validações inicias para execução do processo\n")
        console.print("Obtendo configuração para execução do processo ...\n")
        try:
            config = await get_config_by_name("Ecac_Estadual_SC")
            emails_to = config.conConfiguracao.get("emails")
            user = config.conConfiguracao.get("Usuario")
            password = config.conConfiguracao.get("Senha")

            console.print(
                "Obtendo configuração de email para execução do processo ...\n"
            )
            smtp_config = await get_config_by_name("SMTP")

            smtp_server = smtp_config.conConfiguracao.get("server")
            smtp_user = smtp_config.conConfiguracao.get("user")
            smtp_pass = smtp_config.conConfiguracao.get("password")
            smtp_port = smtp_config.conConfiguracao.get("port")

        except Exception as e:
            console.print(
                f"Erro ao obter as configurações para execução do processo, erro: {e}...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao obter as configurações para execução do processo, erro: {e}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        async with async_playwright() as p:
            browser = await p.firefox.launch(headless=False)

            console.print("Iniciando o browser\n")
            context = await browser.new_context(accept_downloads=True)
            page = await context.new_page()

            await asyncio.sleep(3)

            console.print("Navegando até o site SAT... \n")
            await page.goto(
                "https://sat.sef.sc.gov.br/tax.NET/Login.aspx?ReturnUrl=%2ftax.NET%2fsat.dtec.web%2fSATautenticacao.aspx"
            )
            console.print("Realizando login... \n")
            await asyncio.sleep(3)
            await page.get_by_placeholder("Usuário").fill(user)
            await page.get_by_placeholder("Senha").fill(password)
            await page.get_by_role("link", name="Entrar", exact=True).click()
            await asyncio.sleep(3)
            console.print("Sucesso ao realizar o login... \n")
            await page.get_by_role("link", name="person").click()
            await asyncio.sleep(3)
            console.print("Navegando até caixa de entrada... \n")
            await page.get_by_role("link", name=" Caixa de entrada").click()

            console.print("Extraindo mensagens... \n")
            await page.wait_for_selector('//*[@id="ext-gen47"]')
            time.sleep(3)

            div_content = await page.locator('//*[@id="ext-gen47"]').inner_html()

            console.print("Formatando saida de dados... \n")
            soup = BeautifulSoup(div_content, "html.parser")

            rows = soup.select("tr")
            formatted_output = ""
            corpo_email = ""
            for row in rows:
                try:
                    id_mensagem = row.select_one(".x-grid3-col-0").text.strip()
                    status_lida = row.select_one(".x-grid3-col-1 img")["alt"].strip()
                    remetente = row.select_one(".x-grid3-col-2").text.strip()
                    tipo_mensagem = row.select_one(".x-grid3-col-3").text.strip()
                    data_envio = row.select_one(".x-grid3-col-4").text.strip()
                    assunto = row.select_one(".x-grid3-col-ASSUNTO").text.strip()

                    if status_lida == "Não Lida":
                        formatted_output += (
                            f"ID Mensagem: {id_mensagem}\n"
                            f"Status: {status_lida}\n"
                            f"Remetente: {remetente}\n"
                            f"Tipo de Mensagem: {tipo_mensagem}\n"
                            f"Data de Envio: {data_envio}\n"
                            f"Assunto: {assunto}\n"
                        )
                except:
                    pass

            console.print("Enviando email para area de negócio \n")
            if formatted_output != "":
                corpo_email = f""" Caros(as), \nEspero que este e-mail o encontre bem! \n\n Abaixo, segue um resumo baseado na consulta à caixa postal referente ao estado de SC.
                \n Resultado da consulta: \n {formatted_output} \n """
            else:
                corpo_email = f""" Caros(as), \nEspero que este e-mail o encontre bem! \n\n O processo foi executado para coleta de mensagens na caixa postal referente ao estado de SC, porém não foi encontrado nenhuma nova mensagem."""

            try:
                await context.close()
                await browser.close()
            except Exception as e:
                console.print("Erro ao encerrar o browser... \n", e)

            try:
                send_email(
                    smtp_server,
                    smtp_port,
                    smtp_user,
                    smtp_pass,
                    corpo_email,
                    "Consulta ECAC_Estadual - SC",
                    emails_to,
                )
                log_msg = f"Processo concluído e e-mail disparado para área de negócio"
                console.print(log_msg)
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno=log_msg,
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            except Exception as e:
                log_msg = f"Processo concluído com sucesso, porém houve falha no envio do e-mail para área de negócio"
                console.print(log_msg)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=log_msg,
                    status=RpaHistoricoStatusEnum.Falha,
                )

    except Exception as e:
        log_msg = f"Erro Processo do Ecac Estadual SC: {e}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=log_msg,
            status=RpaHistoricoStatusEnum.Falha,
        )



import asyncio
import getpass
import os
import warnings

import pyautogui
from pywinauto.keyboard import send_keys
from PIL import Image, ImageEnhance
import pytesseract
from pywinauto.application import Application
from pywinauto_recorder.player import set_combobox
from rich.console import Console

from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
    sync_get_config_by_name,
)
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    cod_icms,
    error_after_xml_imported,
    import_nfe,
    importar_notas_outras_empresas,
    is_window_open,
    is_window_open_by_class,
    itens_not_found_supplier,
    kill_process,
    login_emsys,
    rateio_despesa,
    set_variable,
    tipo_despesa,
    type_text_into_field,
    verify_nf_incuded,
    warnings_after_xml_imported,
    worker_sleep,
    zerar_icms,
)
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

pyautogui.PAUSE = 0.5
console = Console()

emsys = EMSys()


async def entrada_de_notas_39(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)

        console.print(config)

        # Seta config entrada na var nota para melhor entendimento
        nota = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Abre um novo emsys
        await kill_process("EMSys")

        #Verifica se a nota ja foi lançada
        console.print("\nVerifica se a nota ja foi lançada...")
        nf_chave_acesso = int(nota.get("nfe"))
        status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
        if status_nf_emsys.get("status") == "Lançada":
            console.print("\nNota ja lançada, processo finalizado...", style="bold green")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Nota já lançada",
                status=RpaHistoricoStatusEnum.Descartado,
            )
        else:
            console.print("\nNota não lançada, iniciando o processo...")
        
        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(1)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(10)

        # Procura campo documento
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(title="Nota Fiscal de Entrada")
        main_window = app["Nota Fiscal de Entrada"]

        console.print(
            "Controles encontrados na janela 'Nota Fiscal de Entrada', navegando entre eles...\n"
        )
        panel_TNotebook = main_window.child_window(
            class_name="TNotebook", found_index=0
        )
        panel_TPage = panel_TNotebook.child_window(class_name="TPage", found_index=0)
        panel_TPageControl = panel_TPage.child_window(
            class_name="TPageControl", found_index=0
        )
        panel_TTabSheet = panel_TPageControl.child_window(
            class_name="TTabSheet", found_index=0
        )
        combo_box_tipo_documento = panel_TTabSheet.child_window(
            class_name="TDBIComboBox", found_index=1
        )

        combo_box_tipo_documento.click()
        console.print(
            "Clique select box, Tipo de documento realizado com sucesso, selecionando o tipo de documento...\n"
        )

        await worker_sleep(4)

        set_combobox("||List", "NFe - NOTA FISCAL ELETRONICA PROPRIA - DANFE SERIE 077")
        console.print(
            "Tipo de documento 'NFe - NOTA FISCAL ELETRONICA PROPRIA - DANFE SERIE 077', selecionado com sucesso...\n"
        )

        await worker_sleep(4)

        # Clica em 'Importar-Nfe'
        imported_nfe = await import_nfe()
        if imported_nfe.sucesso == True:
            console.log(imported_nfe.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=imported_nfe.retorno,
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(10)

        # Move até 'Notas de Outras Empresas'
        pyautogui.press('down', presses=5)
        # Clica em  'OK' para selecionar
        pyautogui.click(970, 666)
        await worker_sleep(3)

        try:
            empresa = str(int(nota.get("cnpjFornecedor")[8:12]))
            max_attempts = 10
            i = 0

            while i < max_attempts:
                try:
                    success = await importar_notas_outras_empresas(
                        nota.get("dataEmissao"), nota.get("numeroNota"), empresa
                    )
                    if success:
                        console.print("Nota importada com sucesso!\n")
                        break
                    else:
                        console.print("Não foi possível importar a nota, tentando novamente...\n")
                except Exception as e:
                    console.print(f"Erro durante a tentativa de importar a nota: {e}\n")

                await worker_sleep(30)
                i += 1

            await worker_sleep(10)

            app = Application().connect(
                title="Informações para importação da Nota Fiscal Eletrônica"
            )
            main_window = app["Informações para importação da Nota Fiscal Eletrônica"]
            
            # Configurar Tipo de Despesa com base no fornecedor e natureza
            fornecedor = nota.get("nomeFornecedor", "").lower()
            natureza = nota.get("natureza", "")
            despesa = None

            if "brs" in fornecedor and natureza == "33":
                despesa = "192"
            elif "ataca" in fornecedor and natureza == "32":
                despesa = "15"
            elif "hygia" in fornecedor and natureza == "34":
                despesa = "193"
            
            if despesa is not None:

                console.print(f"Selecionando Tipo de Despesa: {despesa}...\n")
                tipo_despesa_work = await tipo_despesa(despesa)
                if not tipo_despesa_work.sucesso:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao configurar Tipo de Despesa: {tipo_despesa_work.retorno}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )

                # Configurar ICMS (inserir código e zerar tributação)
                console.print("Configurando ICMS...\n")
                cod_icms_work = await cod_icms("33")
                if not cod_icms_work.sucesso:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao configurar Código ICMS: {cod_icms_work.retorno}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )

                checkbox_zerar_icms = await zerar_icms()
                if not checkbox_zerar_icms.sucesso:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao zerar tributação ICMS: {checkbox_zerar_icms.retorno}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )

                # Selecionar "Manter Natureza de Operação selecionada"
                console.print("Selecionando 'Manter Natureza de Operação selecionada'...\n")
                try:
                    checkbox_natureza = main_window.child_window(
                        title="Manter Natureza de Operação selecionada", class_name="TDBICheckBox"
                    )
                    if not checkbox_natureza.get_toggle_state():
                        checkbox_natureza.click()
                        console.print("'Manter Natureza de Operação selecionada' foi marcado com sucesso.\n")
                except Exception as e:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao selecionar 'Manter Natureza de Operação selecionada': {e}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )

            console.print("Clicando em OK... \n")

            max_attempts = 3
            i = 0
            while i < max_attempts:
                console.print("Clicando no botão de OK...\n")
                try:
                    try:
                        btn_ok = main_window.child_window(title="Ok")
                        btn_ok.click()
                    except:
                        btn_ok = main_window.child_window(title="&Ok")
                        btn_ok.click()
                except:
                    console.print("Não foi possivel clicar no Botão OK... \n")

                await worker_sleep(1)
                i += 1
        except:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao importar nota fiscal.",
                status=RpaHistoricoStatusEnum.Descartado,
            )
        
        await worker_sleep(40)

        await emsys.verify_warning_and_error("Information", "&No")

        await worker_sleep(3)

        # Seleciona pagamento
        console.log("Seleciona Pagamento", style="bold yellow")
        pyautogui.click(623, 374)
        await worker_sleep(1)
        pyautogui.click(889, 349)
        await worker_sleep(1)
        pyautogui.write("27")
        await worker_sleep(1)
        pyautogui.hotkey("enter")

        # Digita "Valor"

        pyautogui.click(1285, 352)
        await worker_sleep(1)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        await worker_sleep(1)
        pyautogui.write(nota["valorNota"])

        await emsys.incluir_registro()
        await worker_sleep(30)

        await worker_sleep(5)
        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")

        await worker_sleep(3)
        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")

        await worker_sleep(2)
        console.print(
            "Verificando a existencia de Warning informando que a Soma dos pagamentos não bate com o valor da nota. ...\n"
        )
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        try:
            warning_pop_up_pagamentos = main_window.child_window(
                class_name="TMessageForm", title="Warning"
            )
        except:
            warning_pop_up_pagamentos = None

        if warning_pop_up_pagamentos.exists():
            console.print(
                "Erro: Warning informando que a Soma dos pagamentos não bate com o valor da nota. ...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"A soma dos pagamentos não bate com o valor da nota.",
                status=RpaHistoricoStatusEnum.Falha,
            )
        else:
            console.print(
                "Warning informando que a Soma dos pagamentos não bate com o valor da nota não existe ...\n"
            )

        max_attempts = 7
        i = 0
        aguarde_rateio_despesa = True

        while i < max_attempts:
            await worker_sleep(3)

            from pywinauto import Desktop

            for window in Desktop(backend="uia").windows():
                if "Rateio" in window.window_text():
                    aguarde_rateio_despesa = False
                    console.print(
                        "A janela 'Rateio da Despesas' foi encontrada. Continuando para andamento do processo...\n"
                    )
                    break

            if not aguarde_rateio_despesa:
                break

            i += 1

        if aguarde_rateio_despesa:
            console.log(f"Número máximo de tentativas atingido. A tela para Rateio da Despesa não foi encontrada.")

        if not aguarde_rateio_despesa:
            despesa_rateio_work = await rateio_despesa(nota.get("filialEmpresaOrigem"))
            if despesa_rateio_work.sucesso == True:
                console.log(despesa_rateio_work.retorno, style="bold green")
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=despesa_rateio_work.retorno,
                    status=RpaHistoricoStatusEnum.Falha,
                )

        # Verifica se a info 'Nota fiscal incluida' está na tela
        await worker_sleep(15)
        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            app = Application().connect(title="Warning")
            main_window = app["Warning"]
            main_window.set_focus()

            console.print(f"Obtendo texto do Warning...\n")
            console.print(f"Tirando print da janela do warning para realização do OCR...\n")

            window_rect = main_window.rectangle()
            screenshot = pyautogui.screenshot(
                region=(
                    window_rect.left,
                    window_rect.top,
                    window_rect.width(),
                    window_rect.height(),
                )
            )
            username = getpass.getuser()
            path_to_png = f"C:\\Users\\{username}\\Downloads\\warning_popup_{nota.get("nfe")}.png"
            screenshot.save(path_to_png)
            console.print(f"Print salvo em {path_to_png}...\n")

            console.print(
                f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
            )
            image = Image.open(path_to_png)
            image = image.convert("L")
            enhancer = ImageEnhance.Contrast(image)
            image = enhancer.enhance(2.0)
            image.save(path_to_png)
            console.print(f"Imagem preparada com sucesso...\n")
            console.print(f"Realizando OCR...\n")
            captured_text = pytesseract.image_to_string(Image.open(path_to_png))
            console.print(
                f"Texto Full capturado {captured_text}...\n"
            )
            os.remove(path_to_png)
            if 'movimento não permitido' in captured_text.lower():
                return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Filial: {nota.get("filialEmpresaOrigem")} está com o livro fechado ou encerrado, verificar com o setor fiscal",
                status=RpaHistoricoStatusEnum.Falha,
                )
            else:
                return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Warning não mapeado para seguimento do robo, mensagem: {captured_text}",
                status=RpaHistoricoStatusEnum.Falha,
                )
            
        await worker_sleep(3)
        retorno = False
        try:
            max_attempts = 60
            i = 0

            while i < max_attempts:
                information_pop_up = await is_window_open("Information")
                if information_pop_up["IsOpened"] == True:
                    break
                else:
                    console.print(f"Aguardando confirmação de nota incluida...\n")
                    await worker_sleep(5)
                    i += 1

            information_pop_up = await is_window_open("Information")
            if information_pop_up["IsOpened"] == True:
                app = Application().connect(class_name="TFrmNotaFiscalEntrada")
                main_window = app["Information"]

                main_window.set_focus()

                console.print(f"Obtendo texto do Information...\n")
                console.print(f"Tirando print da janela do Information para realização do OCR...\n")

                window_rect = main_window.rectangle()
                screenshot = pyautogui.screenshot(
                    region=(
                        window_rect.left,
                        window_rect.top,
                        window_rect.width(),
                        window_rect.height(),
                    )
                )
                username = getpass.getuser()
                path_to_png = f"C:\\Users\\{username}\\Downloads\\information_popup_{nota.get("nfe")}.png"
                screenshot.save(path_to_png)
                console.print(f"Print salvo em {path_to_png}...\n")

                console.print(
                    f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
                )
                image = Image.open(path_to_png)
                image = image.convert("L")
                enhancer = ImageEnhance.Contrast(image)
                image = enhancer.enhance(2.0)
                image.save(path_to_png)
                console.print(f"Imagem preparada com sucesso...\n")
                console.print(f"Realizando OCR...\n")
                captured_text = pytesseract.image_to_string(Image.open(path_to_png))
                console.print(
                    f"Texto Full capturado {captured_text}...\n"
                )
                os.remove(path_to_png)
                if 'nota fiscal inc' in captured_text.lower():
                    console.print(f"Tentando clicar no Botão OK...\n")
                    btn_ok = main_window.child_window(class_name="TButton")

                    if btn_ok.exists():
                        btn_ok.click()
                        retorno = True
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Pop_up Informantion não mapeado para andamento do robô, mensagem {captured_text}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )
            else:
                console.print(f"Aba Information não encontrada")
                retorno = await verify_nf_incuded()

        except Exception as e:
            console.print(f"Erro ao conectar à janela Information: {e}\n")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro em obter o retorno, Nota inserida com sucesso, erro {e}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        if retorno:
            console.print("\nNota lançada com sucesso...", style="bold green")
            await worker_sleep(6)
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno="Nota Lançada com sucesso!",
                status=RpaHistoricoStatusEnum.Sucesso,
            )
        else:
            console.print("Erro ao lançar nota", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao lançar nota",
                status=RpaHistoricoStatusEnum.Falha,
            )

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}

    finally:
        await kill_process("EMSys")



import json
import os
import re
import subprocess
import zipfile
import git
import shutil
import toml
import stat
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload
from pathlib3x import Path
from playwright.async_api import async_playwright
from rich.console import Console

from worker_automate_hub.api.client import sync_get_config_by_name
from worker_automate_hub.config.settings import (
    get_package_version,
    load_env_config,
)
from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from worker_automate_hub.utils.logger import logger

console = Console()


def write_env_config(env_dict: dict, google_credentials: dict):
    try:
        current_dir = Path.cwd()
        assets_path = current_dir / "assets"
        logs_path = current_dir / "logs"
        assets_path.mkdir(exist_ok=True)
        logs_path.mkdir(exist_ok=True)

        config_file_path = current_dir / "settings.toml"
        config_data = {
            "name": "WORKER",
            "params": {
                "api_base_url": env_dict["API_BASE_URL"],
                "api_auth": env_dict["API_AUTHORIZATION"],
                "notify_alive_interval": env_dict["NOTIFY_ALIVE_INTERVAL"],
                "version": get_package_version("worker-automate-hub"),
                "log_level": env_dict["LOG_LEVEL"],
                "drive_url": env_dict["DRIVE_URL"],
                "xml_default_folder": env_dict["XML_DEFAULT_FOLDER"],
                "git_assets_url": env_dict["GIT_ASSETS_URL"],
            },
            "google_credentials": google_credentials["content"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Arquivo de configuração do ambiente criado em {config_file_path}"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="green")

        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao criar o arquivo de configuração do ambiente. Comando retornou: {e}"
        logger.error(err_msg)
        return {
            "Message": err_msg,
            "Status": False,
        }


def add_worker_config(worker):
    try:
        current_dir = Path.cwd()
        config_file_path = current_dir / "settings.toml"

        if not config_file_path.exists():
            raise FileNotFoundError(
                f"O arquivo de configuração não foi encontrado em: {config_file_path}"
            )

        with open(config_file_path, "r") as config_file:
            config_data = toml.load(config_file)

        config_data["id"] = {
            "worker_uuid": worker["uuidRobo"],
            "worker_name": worker["nomRobo"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Informações do worker adicionadas ao arquivo de configuração em {config_file_path}"
        console.print(f"\n{log_msg}\n", style="green")
        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao adicionar informações do worker ao arquivo de configuração.\n Comando retornou: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        return {
            "Message": err_msg,
            "Status": False,
        }


async def install_playwright():
    try:
        result1 = subprocess.run(
            ["pipx", "install", "playwright"],
            check=True,
            capture_output=True,
            text=True,
        )
        logger.info(result1.stdout)
        result2 = subprocess.run(
            ["playwright", "install"], check=True, capture_output=True, text=True
        )
        logger.info("Playwright instalado com sucesso!")
        logger.info(result2.stdout)
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Playwright: {e}")


def extract_zip(zip_path, extract_to):
    """
    Extrai um arquivo ZIP para um diretório especificado.

    :param zip_path: Caminho para o arquivo ZIP.
    :param extract_to: Diretório onde os arquivos serão extraídos.
    """
    if not os.path.exists(extract_to):
        os.makedirs(extract_to)

    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        zip_ref.extractall(extract_to)
        console.print(f"\nArquivos extraídos para {extract_to}\n", style="green")


def extract_drive_folder_id(url: str) -> str:
    """
    Extrai o ID da pasta do Google Drive a partir de uma URL.

    Args:
        url (str): URL da pasta no Google Drive.

    Returns:
        str: O ID da pasta ou uma exceção ValueError se o ID não for encontrado.
    """
    match = re.search(r"folders/([a-zA-Z0-9-_]+)", url)
    if match:
        return match.group(1)
    else:
        raise ValueError("ID da pasta não encontrado na URL.")
    
def handle_remove_readonly(func, path, exc_info):
    # Remove o atributo de somente leitura e tenta novamente
    os.chmod(path, stat.S_IWRITE)
    func(path)

def clone_public_repo(repo_url: str, local_path: str):
    try:
        console.print(f"Clonando o repositório {repo_url} para {local_path}...")
        git.Repo.clone_from(repo_url, local_path)
        console.print(f"Repositório clonado com sucesso em {local_path}.", style="green")
    except Exception as e:
        console.print(f"Erro ao clonar o repositório: {e}", style="red")

def update_assets_v2():
    """
    Atualiza os assets baixando os arquivos do repositório e substituindo os existentes.
    """
    console.print("\nAtualizando os assets...\n", style="bold blue")
    try:
        env_config, _ = load_env_config()
        current_dir = f"{Path.cwd()}\\assets"
        repo_url = env_config["GIT_ASSETS_URL"]
        
        if not os.path.exists(current_dir):
            # Faz o clone do repositório
            console.print(f"Clonando o repositório {repo_url} para {current_dir}...", style="blue")
            clone_public_repo(repo_url, current_dir)

            console.print(
                "\nTodos os arquivos e pastas foram baixados e salvos na pasta 'assets' com sucesso.\n",
                style="bold green",
            )
        else:
            try:
                console.print(f"Atualizando o repositório em {current_dir}...")
                repo = git.Repo(current_dir)
                origin = repo.remotes.origin
                origin.fetch()  
                origin.pull()   
                console.print(f"Repositório atualizado com sucesso em {current_dir}.", style="green")   
            except:
                console.print(f"Falha ao tentar atualizar, iniciando a limpeza...", style="red")
                shutil.rmtree(current_dir, onerror=handle_remove_readonly)

                console.print(f"Clonando o repositório {repo_url} para {current_dir}...", style="blue")
                clone_public_repo(repo_url, current_dir)

                console.print(
                    "\nTodos os arquivos e pastas foram baixados e salvos na pasta 'assets' com sucesso.\n",
                    style="bold green",
                )

        
    except Exception as e:
        console.print(f"\nErro ao atualizar os assets: {e}...\n", style="bold red")


async def install_tesseract(setup_path: Path):
    try:
        # Comando para executar com elevação
        command = f'start-process "{setup_path}" -Verb runAs'

        # Executar o comando usando PowerShell
        result = subprocess.run(
            ["powershell", "-Command", command], capture_output=True
        )
        logger.info(result.stdout)
        logger.info("Tesseract instalado com sucesso!")
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Tesseract: {e}")


async def download_tesseract():
    # Diretório de execução atual
    current_dir = Path.cwd()
    output_folder = current_dir / "temp"
    destination_path = output_folder / "tesseract.exe"

    folder_url = "https://github.com/UB-Mannheim/tesseract/wiki"
    async with async_playwright() as p:
        browser = await p.chromium.launch(
            headless=True
        )  # Defina headless=True se não quiser ver o navegador
        context = await browser.new_context()

        # Abrir uma nova página
        page = await context.new_page()

        # Navegar para a URL do arquivo no Google Drive
        await page.goto(folder_url)

        # Aguarde o carregamento da página e clique no botão de download
        await page.wait_for_selector(
            "#wiki-body > div > ul:nth-child(7) > li > a",
            timeout=60000,
        )
        await page.click("#wiki-body > div > ul:nth-child(7) > li > a")

        # Aguarde o download começar
        download = await page.wait_for_event("download")

        # Salve o arquivo no destino especificado
        await download.save_as(destination_path)

        await browser.close()

    await install_tesseract(destination_path)



import aiohttp

from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.models.dao.rpa_historico import RpaHistorico
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoRequestDTO,
)
from worker_automate_hub.utils.logger import logger


async def store(data: RpaHistoricoRequestDTO) -> dict:
    """
    Armazena o histórico de um processo com status Processando.

    Recebe um RpaHistoricoRequestDTO como parâmetro e salva o
    histórico com status Processando. Retorna um dicionário com o uuid do
    histórico salvo.

    Args:
        data (RpaHistoricoRequestDTO): O histórico a ser salvo.

    Returns:
        RpaHistorico: Dicionário com o uuid do histórico salvo.
    """
    env_config, _ = load_env_config()

    if not data:
        raise ValueError("Parâmetro data deve ser informado")

    if not isinstance(data, RpaHistoricoRequestDTO):
        raise TypeError("Parâmetro data deve ser do tipo RpaHistoricoRequestDTO")

    headers_basic = {
        "Authorization": f"Basic {env_config["API_AUTHORIZATION"]}",
        "Content-Type": "application/json",
    }
    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            payload = data.model_dump_json()

            async with session.post(
                f"{env_config["API_BASE_URL"]}/historico",
                data=payload,
                headers=headers_basic,
            ) as response:
                response_text = await response.text()
                logger.info(f"Resposta store: {response_text}")

                if response.status == 200:
                    try:
                        response_data = await response.json()
                        return RpaHistorico(**response_data)
                    except aiohttp.ContentTypeError:
                        return {
                            "error": "Resposta não é JSON",
                            "status_code": response.status,
                        }
                else:
                    return {"error": response_text, "status_code": response.status}
    except aiohttp.ClientError as e:
        logger.error(f"Erro de cliente aiohttp: {str(e)}")
        return {"error": str(e), "status_code": 500}
    except Exception as e:
        logger.error(f"Erro inesperado: {str(e)}")
        return {"error": str(e), "status_code": 500}


async def update(data: RpaHistoricoRequestDTO) -> dict:
    """
    Atualiza um registro de histórico com base no uuidHistorico informado.

    Args:
        data (RpaHistoricoRequestDTO): Os dados do histórico a ser atualizado.

    Returns:
        RpaHistorico: O histórico atualizado.
    """
    env_config, _ = load_env_config()
    headers_basic = {
        "Authorization": f"Basic {env_config["API_AUTHORIZATION"]}",
        "Content-Type": "application/json",
    }
    if not data or not isinstance(data, RpaHistoricoRequestDTO):
        raise TypeError("Parâmetro data deve ser do tipo RpaHistoricoRequestDTO")
    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            if not data.uuidHistorico:
                raise ValueError("Parâmetro uuidHistorico deve ser informado")

            payload = data.model_dump_json()

            async with session.put(
                f"{env_config["API_BASE_URL"]}/historico",
                data=payload,
                headers=headers_basic,
            ) as response:
                response_text = await response.text()
                logger.info(f"Resposta update: {response_text}")

                if response.status == 200:
                    try:
                        response_data = await response.json()
                        return RpaHistorico(**response_data)
                    except aiohttp.ContentTypeError:
                        return {
                            "error": "Resposta não é JSON",
                            "status_code": response.status,
                        }
                else:
                    return {"error": response_text, "status_code": response.status}
    except aiohttp.ClientError as e:
        logger.error(f"Erro de cliente aiohttp: {str(e)}")
        return {"error": str(e), "status_code": 500}
    except ValueError as e:
        logger.error(f"Erro de valor: {str(e)}")
        return {"error": str(e), "status_code": 400}
    except Exception as e:
        logger.error(f"Erro inesperado: {str(e)}")
        return {"error": str(e), "status_code": 500}



from plyer import notification
from win10toast import ToastNotifier


def task_bar_toast(title: str, message: str, app_name: str, timeout: int = 2):

    notification.notify(
        title=title, message=message, app_name=app_name, timeout=timeout
    )


def show_toast(titulo: str, mensagem: str, icone: str = None, duracao: int = 10):
    """
    Exibe uma notificação na barra de tarefas do Windows.

    :param titulo: Título da notificação.
    :param mensagem: Mensagem da notificação.
    :param icone: Caminho para o arquivo .ico (ícone). Use None para não exibir ícone.
    :param duracao: Tempo em segundos que a notificação ficará visível.
    """
    toaster = ToastNotifier()

    toaster.show_toast(
        titulo,  # Título da notificação
        mensagem,  # Mensagem da notificação
        icon_path=icone,  # Caminho do ícone (ou None)
        duration=duracao,  # Duração da notificação
        threaded=True,  # Para não travar a execução do script
    )

    # Mantém o programa rodando enquanto a notificação está ativa
    while toaster.notification_active():
        pass



import asyncio
import warnings
from datetime import datetime

import pyautogui
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.decorators.timeit import timeit
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.toast import show_toast
from worker_automate_hub.utils.util import (
    api_simplifica,
    extract_nf_number,
    faturar_pre_venda,
    find_element_center,
    find_target_position,
    kill_process,
    login_emsys,
    set_variable,
    take_screenshot,
    take_target_position,
    type_text_into_field,
    wait_window_close,
    worker_sleep,
)

console = Console()

ASSETS_BASE_PATH = "assets/descartes_transferencias_images/"
ALMOXARIFADO_DEFAULT = "50"


@timeit
async def descartes(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        # Inicializa variaveis
        nota_fiscal = [None]
        log_msg = None
        valor_nota = None
        # Get config from BOF
        console.print("Obtendo configuração...\n")
        config = await get_config_by_name(name="Descartes_Emsys")
        # Popula Variaveis
        itens = task.configEntrada.get("itens", [])
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()

        # Print da resolução
        console.print(f"Largura: {screen_width}, Altura: {screen_height}")

        # Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            console.print("Pesquisando por: Cadastro Pré Venda")
            type_text_into_field(
                "Cadastro Pré Venda", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(1)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Cadastro Pre Venda' realizada com sucesso",
                style="bold green",
            )
            await worker_sleep(3)
            try:
                app = Application().connect(class_name="TFrmSelecionaTipoPreVenda")
                select_prevenda_type = app["Selecione o Tipo de Pré-Venda"]

                if select_prevenda_type.exists():
                    tipo = select_prevenda_type.child_window(class_name="TComboBox", found_index=0)
                    tipo.select("Orçamento")
                    confirm = select_prevenda_type.child_window(class_name="TDBIBitBtn", found_index=1)
                    confirm.click()
            except:
                console.print("Sem tela de selecionar modelo de pre venda", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(7)

        # Preenche data de validade
        console.print("Preenchendo a data de validade...\n")
        screenshot_path = take_screenshot()
        target_pos = (
            961,
            331,
        )  # find_target_position(screenshot_path, "Validade", 10, 0, 15)
        if target_pos == None:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel encontrar o campo de validade",
                status=RpaHistoricoStatusEnum.Falha,
            )

        pyautogui.click(target_pos)
        pyautogui.write(f'{datetime.now().strftime("%d/%m/%Y")}', interval=0.1)
        pyautogui.press("tab")
        console.print(
            f"\nValidade Digitada: '{datetime.now().strftime("%d/%m/%Y")}'\n",
            style="bold green",
        )
        await worker_sleep(1)

        # Condição da Pré-Venda
        console.print("Selecionando a Condição da Pré-Venda\n")
        condicao_field = find_target_position(screenshot_path, "Condição", 10, 0, 15)
        if condicao_field == None:
            condicao_field = (1054, 330)

        pyautogui.click(condicao_field)
        await worker_sleep(1)
        pyautogui.write("A")
        await worker_sleep(1)
        pyautogui.press("down")
        pyautogui.press("enter")
        await worker_sleep(1)

        # Preenche o campo do cliente com o número da filial
        console.print("Preenchendo o campo do cliente com o número da filial...\n")
        cliente_field_position = await find_element_center(
            ASSETS_BASE_PATH + "field_cliente.png", (795, 354, 128, 50), 10
        )
        if cliente_field_position == None:
            cliente_field_position = (884, 384)

        pyautogui.click(cliente_field_position)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        pyautogui.write(task.configEntrada["filialEmpresaOrigem"])
        pyautogui.hotkey("tab")
        await worker_sleep(10)

        try:
            # Verificando a existência da janela Busca representante e clicando em Cancelar
            console.print(
                "Verificando a existência da janela Busca representante e clicando em Cancelar...\n"
            )
            app = Application().connect(class_name="TFrmBuscaGeralDialog")
            app = app["TFrmBuscaGeralDialog"]
            btn_cancelar = app.child_window(title="&Cancelar", class_name="TBitBtn")
            if btn_cancelar.exists():
                app.set_focus()
                btn_cancelar.click_input()
                console.print(f" botão Cancelar clicado com sucesso")
            else:
                console.print(f"Botão Cancelar Não encontrado")

        except:
            console.log(
                f"Nenhuma tela de Busca representante foi encontrada.",
                style="bold green",
            )

        await worker_sleep(2)

        # Verifica se precisa selecionar endereço
        console.print("Verificando se precisa selecionar endereço...\n")
        screenshot_path = take_screenshot()
        window_seleciona_endereco_position = take_target_position(
            screenshot_path, "Endereço"
        )
        if window_seleciona_endereco_position is not None:
            log_msg = f"Aviso para selecionar Endereço | Número da nota: {nota_fiscal} | Valor: {valor_nota}"
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "ERRO",
                log_msg,
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
            )
        else:
            log_msg = f"Sem Aviso de Seleção de Endereço Número da nota: {nota_fiscal} | Valor: {valor_nota}"
            console.print(log_msg, style="bold green")
            logger.info(log_msg)

        await worker_sleep(5)

        # Aviso "Deseja alterar a condição de pagamento informada no cadastro do cliente?"
        console.print(
            "Verificando alerta de alteração de pagamento informada no cadastro do cliente...\n"
        )
        # screenshot_path = take_screenshot()
        # payment_condition_warning_position = take_target_position(screenshot_path, "pagamento")
        # if payment_condition_warning_position is not None:
        button_no_position = (
            1000,
            568,
        )  # find_target_position(screenshot_path, "No", attempts=15)
        pyautogui.click(button_no_position)
        console.print(
            f"\nClicou 'No' Mensagem 'Deseja alterar a condição de pagamento informada no cadastro do cliente?'",
            style="bold green",
        )
        await worker_sleep(10)
        # else:
        #     log_msg = f"\nError Message: Aviso de condição de pagamento não encontrado"
        #     logger.info(log_msg)
        #     console.print(log_msg, style="bold red")

        # Seleciona 'Custo Médio' (Seleção do tipo de preço)
        console.print("Seleciona 'Custo Médio' (Seleção do tipo de preço)...\n")
        custo_medio_select_position = (851, 523)
        pyautogui.click(custo_medio_select_position)
        await worker_sleep(1)
        button_ok_position = (1042,583)
        pyautogui.click(button_ok_position)
        await worker_sleep(1)
        console.print(f"\nClicou OK 'Custo médio'", style="bold green")

        await worker_sleep(5)

        # Clica em ok na mensagem "Existem Pré-Vendas em aberto para este cliente."
        console.print("Clica em ok na mensagem 'Existem Pré-Vendas em aberto para este cliente.'\n")
        screenshot_path = take_screenshot()
        existing_pre_venda_position = find_target_position(
            screenshot_path, "Existem", attempts=15
        )

        if existing_pre_venda_position == None:
            existing_pre_venda_position = await find_element_center(
                ASSETS_BASE_PATH + "existing_pre_venda.png", (831, 437, 247, 156), 15
            )

        if existing_pre_venda_position is not None:
            button_ok_position = (962, 562)
            pyautogui.click(button_ok_position)
            console.print(f"\nClicou OK 'Pre Venda Existente'", style="bold green")
            await worker_sleep(5)
        else:
            log_msg = f"\nError Message: Menssagem de prevenda existente não encontrada"
            logger.info(log_msg)
            console.print(log_msg, style="bold yellow")

        # Define representante para "1"
        console.print("Definindo representante para '1'\n")
        screenshot_path = take_screenshot()
        field_representante_position = find_target_position(
            screenshot_path, "Representante", 0, 50, attempts=15
        )

        if field_representante_position == None:
            field_representante_position = await find_element_center(
                ASSETS_BASE_PATH + "field_representante.png", (679, 416, 214, 72), 15
            )
            if field_representante_position is not None:
                lista = list(field_representante_position)
                lista[0] += 50
                lista[1] += 1
                field_representante_position = tuple(lista)

        if field_representante_position is not None:
            pyautogui.doubleClick(field_representante_position)
            pyautogui.hotkey("ctrl", "a")
            pyautogui.hotkey("del")
            pyautogui.write("1")
            pyautogui.hotkey("tab")
            # pyautogui.move(789, 523)
        await worker_sleep(5)

        # Seleciona modelo de capa
        console.print("Selecionando o modelo de capa...")
        # screenshot_path = take_screenshot()
        # model_descarte_position = find_target_position(
        #     screenshot_path, "Modelo", 0, 100, attempts=8
        # )

        # if model_descarte_position == None:
        #     model_descarte_position = await find_element_center(
        #         ASSETS_BASE_PATH + "field_modelo_faturamento.png",
        #         (681, 489, 546, 96),
        #         15,
        #     )
        #     if model_descarte_position is not None:
        #         lista = list(model_descarte_position)
        #         lista[0] += 100
        #         model_descarte_position = tuple(lista)

        # if model_descarte_position == None:
        model_descarte_position = (848, 527)

        if model_descarte_position is not None:
            # pyautogui.click(848, 527)
            pyautogui.click(model_descarte_position)
            pyautogui.click(1500, 800)
            pyautogui.write("B")
            pyautogui.hotkey("tab")
        else:
            log_msg = f"Campo Modelo na capa da nota não encontrado | Número da nota: {nota_fiscal} | Valor: {valor_nota}"
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "ERRO",
                log_msg,
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
            )

        # Abre Menu itens
        console.print("Abrindo Menu Itens...\n")
        menu_itens = await find_element_center(
            ASSETS_BASE_PATH + "menu_itens.png", (526, 286, 152, 45), 10
        )

        if menu_itens == None:
            menu_itens = (570, 317)

        if menu_itens is not None:
            pyautogui.click(menu_itens)
        else:
            log_msg = f'Campo "Itens" no menu da pré-venda não encontrado. | Número da nota: {nota_fiscal} | Valor: {valor_nota}'
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "ERRO",
                log_msg,
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
            )

        await worker_sleep(2)

        # Loop de itens
        console.print("Inicio do loop de itens\n")

        for item in itens:
            screenshot_path = take_screenshot()
            # Clica no botão inclui para abrir a tela de item
            console.print("Clicando em Incluir...\n")
            button_incluir = (
                905,
                573,
            )  # find_target_position(screenshot_path, "Incluir", 0, 0, attempts=15)
            if button_incluir is not None:
                pyautogui.click(button_incluir)
                console.print("\nClicou em 'Incluir'", style="bold green")
            else:
                log_msg = f'Botão "Incluir" não encontrado | Número da nota: {nota_fiscal} | Valor: {valor_nota}'
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    log_msg,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
                )
            await worker_sleep(3)

            # Digita Almoxarifado
            console.print("Preenchendo o campo de almoxarifado...\n")
            screenshot_path = take_screenshot()
            field_almoxarifado = (
                839,
                313,
            )  # find_target_position(screenshot_path, "Almoxarifado",0, 129, 15)
            if field_almoxarifado is not None:
                pyautogui.doubleClick(field_almoxarifado)
                pyautogui.hotkey("del")
                pyautogui.write(
                    task.configEntrada["filialEmpresaOrigem"] + ALMOXARIFADO_DEFAULT
                )
                pyautogui.hotkey("tab")
                await worker_sleep(2)
                console.print(
                    f"\nDigitou almoxarifado {task.configEntrada["filialEmpresaOrigem"] + ALMOXARIFADO_DEFAULT}",
                    style="bold green",
                )
            else:
                log_msg = f"Campo Almoxarifado não encontrado | Número da nota: {nota_fiscal} | Valor: {valor_nota}"
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    log_msg,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
                )

            # Segue para o campo do item
            console.print("Preenchendo o campo do item...\n")
            field_item = (
                841,
                339,
            )  # find_target_position(screenshot_path, "Item", 0, 130, 15)
            pyautogui.doubleClick(field_item)
            pyautogui.hotkey("del")
            pyautogui.write(item["codigoProduto"])
            pyautogui.hotkey("tab")
            await worker_sleep(2)
            console.print(f"\nDigitou item {item['codigoProduto']}", style="bold green")

            # Checa tela de pesquisa de item
            console.print("Verificando a existencia da tela de pesquisa de item...\n")
            screenshot_path = take_screenshot()
            window_pesquisa_item = await find_element_center(
                ASSETS_BASE_PATH + "window_pesquisa_item.png", (488, 226, 352, 175), 10
            )
            console.print(
                f"Produto {item['codigoProduto']} encontrado", style="bold green"
            )
            logger.info(f"Produto {item['codigoProduto']} encontrado")

            if window_pesquisa_item is not None:
                observacao = (
                    f"Item {item['codigoProduto']} não encontrado, verificar cadastro"
                )
                console.print(f"{observacao}", style="bold green")
                logger.info(f"{observacao}")
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    observacao,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=observacao,
                    status=RpaHistoricoStatusEnum.Falha,
                )

            # Checa se existe alerta de item sem preço, se existir retorna erro(simplifica e bof)
            console.print(
                "Verificando se existe alerta de item sem preço, se existir retorna erro(simplifica e bof)...\n"
            )
            warning_price = await find_element_center(
                ASSETS_BASE_PATH + "warning_item_price.png", (824, 426, 255, 191), 10
            )
            if warning_price is not None:
                observacao = f"Item {item['codigoProduto']} não possui preço, verificar erro de estoque ou de bloqueio."
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    observacao,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=observacao,
                    status=RpaHistoricoStatusEnum.Falha,
                )

            screenshot_path = take_screenshot()

            await worker_sleep(2)

            # Seleciona o Saldo Disponivel e verifica se ah possibilidade do descarte
            console.print(
                "Selecionando o Saldo Disponivel e verificando se há possibilidade do descarte...\n"
            )
            try:
                app = Application().connect(title="Inclui Item Pré Venda")
                item_pre_venda = app["Inclui Item Pré Venda"]
                saldo_disponivel: str = item_pre_venda.child_window(
                    class_name="TDBIEditNumber", found_index=9
                ).window_text()
            except Exception as error:
                console.print(f"Erro ao selecionar o Saldo Disponivel: {error}")
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    error,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=error,
                    status=RpaHistoricoStatusEnum.Falha,
                )

            try:
                # Remove parênteses, se houver
                saldo_disponivel = saldo_disponivel.strip("()")

                # Caso tenha vírgula e ponto, tratamos como separador de milhar e decimal
                if "." in saldo_disponivel and "," in saldo_disponivel:
                    saldo_disponivel = saldo_disponivel.replace(".", "").replace(
                        ",", "."
                    )
                else:
                    # Caso contrário, apenas troca a vírgula por ponto para converter o decimal
                    saldo_disponivel = saldo_disponivel.replace(",", ".")

                amount_avaliable = float(saldo_disponivel)

                console.print(
                    f"Saldo Disponivel: '{amount_avaliable}'", style="bold green"
                )
            except Exception as error:
                console.print(f"Erro ao converter o Saldo Disponível: {error}")
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    error,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=error,
                    status=RpaHistoricoStatusEnum.Falha,
                )

            # Verifica se o saldo disponivel é valido para descartar
            if int(amount_avaliable) > 0 and int(amount_avaliable) >= int(item["qtd"]):
                field_quantidade = (
                    1047,
                    606,
                )  # find_target_position(screenshot_path, "Quantidade", 20, 0, 15)
                pyautogui.doubleClick(field_quantidade)
                pyautogui.hotkey("del")
                pyautogui.write(str(item["qtd"]))
                pyautogui.hotkey("tab")
                await worker_sleep(2)
            else:
                log_msg = f"Saldo disponivel: '{amount_avaliable}' é menor que '{item['qtd']}' o valor que deveria ser descartado. Item: '{item['codigoProduto']}'"
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    log_msg,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                console.print(log_msg, style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
                )

            # Clica em incluir para adicionar o item na nota
            console.print("Clicando em incluir para adicionar o item na nota...\n")
            button_incluir_item = (
                1007,
                745,
            )  # find_target_position(screenshot_path, "Inlcuir", 0, 0, 15)
            if button_incluir_item is not None:
                pyautogui.click(button_incluir_item)
                await worker_sleep(2)
            else:
                log_msg = f"Botao 'Incluir' item não encontrado"
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    log_msg,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                console.print(log_msg, style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=log_msg,
                    status=RpaHistoricoStatusEnum.Falha,
                )

            try:
                # Verificar tela de Valor de custo maior que preço de venda do item
                console.print("Verificando tela de Valor de custo...\n")
                custo_window = Application().connect(title="Warning")
                custo_window = custo_window["Warning"]

                text_custo = custo_window.window_text()
                if "Warning" in text_custo:
                    log_msg = f"O valor de custo do Item: {item['codigoProduto']} é maior que o valor de venda."
                    await api_simplifica(
                        task.configEntrada["urlRetorno"],
                        "ERRO",
                        log_msg,
                        task.configEntrada["uuidSimplifica"],
                        nota_fiscal,
                        valor_nota,
                    )
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=log_msg,
                        status=RpaHistoricoStatusEnum.Falha,
                    )
            except:
                console.log(
                    "Nenhuma tela de warning foi encontrada", style="bold green"
                )

            try:
                # Verificando janela Confirm - Item já incluido nesta pré-venda, Deseja continuar?
                console.print(
                    "Verificando janela Confirm - Item já incluido nesta pré-venda, Deseja continuar?...\n"
                )
                custo_window = Application().connect(title="Confirm")
                custo_window = custo_window["Confirm"]

                text_custo = custo_window.window_text()
                if "Confirm" in text_custo:
                    btn_window = custo_window["&Yes"]
                    if btn_window.exists():
                        btn_window.click()
                        console.print(f" botão Yes clicado com sucesso")
                    else:
                        console.print(f"Botão Yes Não encontrado")

            except:
                console.log(
                    "Nenhuma tela de Confirm - Item já incluido nesta pré-venda foi encontrada",
                    style="bold green",
                )

            # Clica em cancelar para fechar a tela e abrir novamente caso houver mais itens
            console.print(
                "Clicando em cancelar para fechar a tela e abrir novamente caso houver mais itens...\n"
            )
            button_cancela_item = (
                1194,
                745,
            )  # find_target_position(screenshot_path, "Cancela", 0, 0, 15)
            if button_cancela_item is not None:
                pyautogui.click(button_cancela_item)
                await worker_sleep(2)
            else:
                log_msg = f"Botao cancelar para fechar a tela do item nao encontrado"
                await api_simplifica(
                    task.configEntrada["urlRetorno"],
                    "ERRO",
                    log_msg,
                    task.configEntrada["uuidSimplifica"],
                    nota_fiscal,
                    valor_nota,
                )
                console.print(log_msg, style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
                )

        await worker_sleep(2)

        # Clica no botão "+" no canto superior esquerdo para lançar a pre-venda
        console.print(
            "Clica no botão '+' no canto superior esquerdo para lançar a pre-venda"
        )
        # Precisa manter por imagem pois não tem texto
        button_lanca_pre_venda = await find_element_center(
            ASSETS_BASE_PATH + "button_lanca_prevenda.png", (490, 204, 192, 207), 15
        )
        if button_lanca_pre_venda is not None:
            pyautogui.click(button_lanca_pre_venda.x, button_lanca_pre_venda.y)
            console.print("\nLançou Pré-Venda", style="bold green")
        else:
            log_msg = f"Botao lança pre-venda nao encontrado"
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "ERRO",
                log_msg,
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
            )

        await worker_sleep(15)

        # Verifica mensagem de "Pré-Venda incluida com número: xxxxx"
        console.print(
            "Verificando mensagem de 'Pré-Venda incluida com número: xxxxx'...\n"
        )
        # Clica no ok da mensagem
        try:
            app = Application().connect(title="Informação")
            dialog = app.window(title="Informação")

            btn_ok = dialog.child_window(title="OK", class_name="Button")
            if btn_ok.exists():
                try:
                    btn_ok.click()
                    await worker_sleep(3)
                    console.print(
                        "O botão OK de pré-venda incluída foi clicado com sucesso.",
                        style="green",
                    )
                except Exception as e:
                    console.print(
                        f"Falha ao clicar no botão OK de pré-venda incluída: {e}",
                        style="red",
                    )
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Falha ao clicar no botão OK de pré-venda incluída: {e}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )

            else:

                console.print(
                    "O Botão OK de pré-venda incluída não foi encontrado.", style="red"
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="O Botão OK de pré-venda incluída não foi encontrado.",
                    status=RpaHistoricoStatusEnum.Falha,
                )

        except Exception as e:
            console.print(
                f"O Botão OK de pré-venda incluída não foi encontrado: {e}", style="red"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"O Botão OK de pré-venda incluída não foi encontrado: {e}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        screenshot_path = take_screenshot()

        # Message 'Deseja pesquisar pré-venda?'
        console.print(
            "Verificando a existencia da mensagem: 'Deseja pesquisar pré-venda?'...\n"
        )
        message_prevenda = take_target_position(screenshot_path, "Deseja")
        if message_prevenda is not None:
            button_yes = find_target_position(screenshot_path, "Yes", attempts=15)
            pyautogui.click(button_yes)
        else:
            log_msg = f"Mensagem 'Deseja pesquisar pré-venda?' não encontrada."
            console.print(log_msg, style="bold yellow")

        screenshot_path = take_screenshot()
        # Confirma pré-venda
        # Pode não precisar em descartes, mas em trânsferencias é obrigatório
        console.print("Confirmando a Pre-Venda...\n")
        button_confirma_transferencia = take_target_position(
            screenshot_path, "confirma"
        )
        if button_confirma_transferencia is not None:
            pyautogui.click(button_confirma_transferencia)
            console.log("Confirmou transferencia", style="bold green")
        else:
            log_msg = f"Botao 'Confirma' não encontrado"
            console.print(log_msg, style="bold yellow")

        pyautogui.moveTo(1200, 300)

        console.print("Verificando a mensagem: Confirmar transferencia...\n")
        await worker_sleep(5)
        pyautogui.moveTo(1200, 300)
        screenshot_path = take_screenshot()
        message_confirma_transferencia = take_target_position(
            screenshot_path, "confirmar"
        )
        if message_confirma_transferencia is not None:
            # clica em sim na mensagem
            screenshot_path = take_screenshot()
            button_yes = find_target_position(screenshot_path, "Yes", attempts=8)
            pyautogui.click(button_yes)
            console.log(
                "Cliclou em 'Sim' para cofirmar a pré-venda", style="bold green"
            )
            await worker_sleep(3)
            pyautogui.click(956, 559)
            pyautogui.moveTo(1200, 300)
            
            await worker_sleep(5)
            screenshot_path = take_screenshot()
            vencimento_message_primeira_parcela = take_target_position(
                screenshot_path, "vencimento"
            )
            # Pode nao aparecer na prod
            if vencimento_message_primeira_parcela is not None:
                button_yes = find_target_position(screenshot_path, "Yes", attempts=15)
                pyautogui.click(button_yes)
            await worker_sleep(5)
            screenshot_path = take_screenshot()
            # Clica no OK 'Pre-Venda incluida com sucesso'
            button_ok = find_target_position(screenshot_path, "Ok", attempts=15)
            pyautogui.click(button_ok)
            console.log(
                "Cliclou em 'OK' para pré-venda confirmada com sucesso",
                style="bold green",
            )
        else:
            log_msg = (
                f"Mensagem 'Deseja realmente confirmar esta pré-venda?' não encontrada."
            )
            console.print(log_msg, style="bold yellow")

        pyautogui.moveTo(1000, 500)
        await worker_sleep(10)

        retorno = await faturar_pre_venda(task)
        if retorno["sucesso"] == True:
            console.log(f"Faturou com sucesso!", style="bold green")
            valor_nota = retorno["valor_nota"]
        else:
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "ERRO",
                retorno["retorno"],
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=retorno["retorno"],
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(10)

        # Mensagem de nota fiscal gerada com número
        log_msg = "Extraindo numero da nota fiscal"
        console.log(log_msg, style="bold green")
        logger.info(log_msg)
        nota_fiscal = await extract_nf_number()
        console.print(f"\nNumero NF: '{nota_fiscal}'", style="bold green")

        await worker_sleep(15)

        # Transmitir a nota
        console.print("Transmitindo a nota...\n", style="bold green")
        pyautogui.click(875, 596)
        logger.info("\nNota Transmitida")
        console.print("\nNota Transmitida", style="bold green")

        await worker_sleep(20)

        # aguardando nota ser transmitida
        aguardando_nota = await wait_window_close("Aguarde")
        if aguardando_nota == False:
            # Clica em ok "processo finalizado"
            await worker_sleep(3)
            pyautogui.click(957, 556)
            # Clica em fechar
            await worker_sleep(3)
            pyautogui.click(1200, 667)
            log_msg = f"Nota lançada com sucesso! Número da nota: {nota_fiscal} | Valor: {valor_nota}"
            await api_simplifica(
                task.configEntrada["urlRetorno"],
                "SUCESSO",
                log_msg,
                task.configEntrada["uuidSimplifica"],
                nota_fiscal,
                valor_nota,
            )
            show_toast("Sucesso", f"Processo Descartes finalizado com sucesso")
            return RpaRetornoProcessoDTO(
                sucesso=True, retorno=log_msg, status=RpaHistoricoStatusEnum.Sucesso
            )
        else:
            log_msg = f"Tempo de espera para lançar a nota execedido. Número da nota: {nota_fiscal} | Valor: {valor_nota}"
            console.print(log_msg)
            logger.error(log_msg)
            show_toast("Falha", log_msg)
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
            )

    except Exception as ex:
        log_msg = f"Erro Processo Descartes: {ex} | Número da nota: {nota_fiscal} | Valor: {valor_nota}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        await api_simplifica(
            task.configEntrada["urlRetorno"],
            "ERRO",
            log_msg,
            task.configEntrada["uuidSimplifica"],
            nota_fiscal,
            valor_nota,
        )
        show_toast("Falha", log_msg)

        return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha
        )
    finally:
        await kill_process("EMSys")


import getpass
import warnings
import os

import pyautogui
import pytesseract
from pywinauto.application import Application
from PIL import Image, ImageEnhance
from pywinauto.keyboard import send_keys
from pywinauto_recorder.player import set_combobox
from rich.console import Console
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
)
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    delete_xml,
    error_after_xml_imported,
    extract_group_by_itens,
    get_xml,
    import_nfe,
    incluir_registro,
    is_window_open,
    is_window_open_by_class,
    itens_not_found_supplier,
    kill_process,
    login_emsys,
    read_xml_file,
    select_documento_type,
    set_variable,
    type_text_into_field,
    warnings_after_xml_imported,
    worker_sleep,
    check_nota_importada,
)

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()


async def entrada_de_notas_207(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)

        # Seta config entrada na var nota para melhor entendimento
        nota = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_process("EMSys")

        # Download XML
        console.log("Realizando o download do XML..\n")
        await save_xml_to_downloads(nota["nfe"])

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)

        # Procura campo documento
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        document_type = await select_documento_type(
            "NOTA FISCAL DE ENTRADA ELETRONICA - DANFE"
        )
        if document_type.sucesso == True:
            console.log(document_type.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=document_type.retorno,
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(4)

        # Clica em 'Importar-Nfe'
        imported_nfe = await import_nfe()
        if imported_nfe.sucesso == True:
            console.log(imported_nfe.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=imported_nfe.retorno,
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(5)

        await get_xml(nota.get("nfe"))
        await worker_sleep(3)

        # VERIFICANDO A EXISTENCIA DE WARNINGS
        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            warning_work = await warnings_after_xml_imported()
            if warning_work.sucesso == True:
                console.log(warning_work.retorno, style="bold green")
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=warning_work.retorno,
                    status=RpaHistoricoStatusEnum.Falha,
                )

        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            error_work = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=error_work.sucesso,
                retorno=error_work.retorno,
                status=error_work.status,
            )

        app = Application().connect(
            title="Informações para importação da Nota Fiscal Eletrônica"
        )
        main_window = app["Informações para importação da Nota Fiscal Eletrônica"]

        # INTERAGINDO COM A NATUREZA DA OPERACAO
        cfop = int(nota.get("cfop"))
        console.print(f"Inserindo a informação da CFOP, caso se aplique {cfop} ...\n")
        if cfop == 5655 or str(cfop).startswith("56"):
            combo_box_natureza_operacao = main_window.child_window(
                class_name="TDBIComboBox", found_index=0
            )
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            set_combobox("||List", "1652-COMPRA DE MERCADORIAS- 1.652")
            await worker_sleep(3)

        else:
            console.print(
                "Erro mapeado, CFOP diferente de 5655 ou 56, necessario ação manual ou ajuste no robo...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro mapeado, CFOP diferente de 5655 ou 56, necessario ação manual ou ajuste no robo",
                status=RpaHistoricoStatusEnum.Falha,
            )

        # INTERAGINDO COM O CAMPO ALMOXARIFADO
        filialEmpresaOrigem = nota.get("filialEmpresaOrigem")
        console.print(
            f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...\n"
        )
        # task_bar_toast("Teste toast bar", f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...", 'Worker', 10)
        # show_toast("Teste toast", f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...")
        try:
            new_app = Application(backend="uia").connect(
                title="Informações para importação da Nota Fiscal Eletrônica"
            )
            window = new_app["Informações para importação da Nota Fiscal Eletrônica"]
            edit = window.child_window(
                class_name="TDBIEditCode", found_index=3, control_type="Edit"
            )
            valor_almoxarifado = filialEmpresaOrigem + "50"
            edit.set_edit_text(valor_almoxarifado)
            edit.type_keys("{TAB}")
        except Exception as e:
            console.print(f"Erro ao iterar itens de almoxarifado: {e}")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao iterar itens de almoxarifado: {e}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(3)
        # INTERAGINDO COM CHECKBOX Utilizar unidade de agrupamento dos itens
        console.print("Verificando se a nota é do fornecedor SIM Lubrificantes \n")
        fornecedor = nota["nomeFornecedor"]
        console.print(f"Fornecedor: {fornecedor} ...\n")
        if "sim lubrificantes" in fornecedor.lower():
            console.print(
                f"Sim, nota emitida para: {fornecedor}, marcando o agrupar por unidade de medida...\n"
            )
            checkbox = window.child_window(
                title="Utilizar unidade de agrupamento dos itens",
                class_name="TCheckBox",
                control_type="CheckBox",
            )
            if not checkbox.get_toggle_state() == 1:
                checkbox.click()
                console.print("Realizado o agrupamento por unidade de medida... \n")
        else:
            console.print(
                "Não foi necessario realizar o agrupamento por unidade de medida... \n"
            )

        await worker_sleep(2)
        console.print("Clicando em OK... \n")

        max_attempts = 3
        i = 0
        while i < max_attempts:
            console.print("Clicando no botão de OK...\n")
            try:
                try:
                    btn_ok = main_window.child_window(title="Ok")
                    btn_ok.click()
                except:
                    btn_ok = main_window.child_window(title="&Ok")
                    btn_ok.click()
            except:
                console.print("Não foi possivel clicar no Botão OK... \n")

            await worker_sleep(3)

            console.print(
                "Verificando a existencia da tela Informações para importação da Nota Fiscal Eletrônica...\n"
            )

            try:
                informacao_nf_eletronica = await is_window_open(
                    "Informações para importação da Nota Fiscal Eletrônica"
                )
                if informacao_nf_eletronica["IsOpened"] == False:
                    console.print(
                        "Tela Informações para importação da Nota Fiscal Eletrônica fechada, seguindo com o processo"
                    )
                    break
            except Exception as e:
                console.print(
                    f"Tela Informações para importação da Nota Fiscal Eletrônica encontrada. Tentativa {i + 1}/{max_attempts}."
                )

            i += 1

        if i == max_attempts:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Número máximo de tentativas atingido, Não foi possivel finalizar os trabalhos na tela de Informações para importação da Nota Fiscal Eletrônica",
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(6)

        try:
            console.print(
                "Verificando a existencia de POP-UP de Itens não localizados ou NCM ...\n"
            )
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")
            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))
                if itens_by_supplier_work.get("window") == "NCM":
                    console.log(itens_by_supplier_work.get("retorno"), style="bold green")
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=itens_by_supplier_work.get("retorno"),
                        status=RpaHistoricoStatusEnum.Falha,
                    )
        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(3)
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        console.print("Acessando os itens da nota... \n")
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
        panel_TTabSheet.wait("visible")
        panel_TTabSheet.click()
        send_keys("{DOWN " + ("5") + "}")

        # CONFIRMANDO SE A ABA DE ITENS FOI ACESSADA COM SUCESSO
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TPage.wait("visible")
        panel_TTabSheet = panel_TPage.child_window(class_name="TTabSheet")
        title_n_serie = panel_TPage.child_window(title="N° Série")

        console.print("Verificando se os itens foram abertos com sucesso... \n")
        if not title_n_serie:
            console.print(f"Não foi possivel acessar a aba de 'Itens da nota...\n")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel acessar a aba de 'Itens da nota'",
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(2)

        console.print("Acessando os itens indivualmente... \n")
        send_keys("{TAB 2}", pause=0.1)
        await worker_sleep(2)

        username = getpass.getuser()
        path_to_xml = f"C:\\Users\\{username}\\Downloads\\{nota["nfe"]}.xml"
        get_xml_itens = await read_xml_file(path_to_xml)
        itens = await extract_group_by_itens(get_xml_itens)

        console.print(
            f"Trabalhando com os itens, alterando a unidade com base na descrição do item \n"
        )
        try:
            for item in itens:
                n_item = item["n_item"]
                formato = item["formato"]
                descricao = item["descricao"]
                if formato is None:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Não foi possivel acessar a extrair a quantidade/Unidade de medidade do XML, item: {item}",
                        status=RpaHistoricoStatusEnum.Falha,
                    )
                pos_x = formato.find("X")
                cod_split = str(formato[0:pos_x])

                cod_split = cod_split.replace("(","")

                console.print(f"Item a ser ajustado: {cod_split} \n")

                send_keys("^({HOME})")
                await worker_sleep(1)

                if int(n_item) > 1:
                    send_keys("{DOWN " + str(n_item) + "}")

                if cod_split != "1":
                    console.print(f"Item a ser ajustado: {descricao} \n")
                    await worker_sleep(2)
                    send_keys("+{F10}")
                    await worker_sleep(1)
                    send_keys("{DOWN 2}")
                    await worker_sleep(1)
                    send_keys("{ENTER}")

                    await worker_sleep(2)
                    app = Application().connect(title="Alteração de Item")
                    main_window = app["Alteração de Item"]

                    main_window.set_focus()

                    edit = main_window.child_window(
                        class_name="TDBIEditCode", found_index=0
                    )

                    # ITERAGINDO COM O IPI
                    tpage_ipi = main_window.child_window(
                        class_name="TPanel", found_index=0
                    )
                    ipi = tpage_ipi.child_window(
                        class_name="TDBIComboBox", found_index=2
                    )

                    ipi_value = ipi.window_text()

                    console.print(
                        f"Trabalhando com os itens, valor do IP {ipi_value}... \n"
                    )
                    if len(ipi_value) == 0:
                        console.print(
                            f"Trabalhando com os itens, valor do IP em branco, selecionando IPI 0% ... \n"
                        )
                        ipi.click_input()
                        send_keys("^({HOME})")
                        send_keys("{DOWN 6}")
                        send_keys("{ENTER}")

                        await worker_sleep(4)
                        tpage_ipi = main_window.child_window(
                            class_name="TPanel", found_index=0
                        )
                        ipi = tpage_ipi.child_window(
                            class_name="TDBIComboBox", found_index=2
                        )

                        ipi_value = ipi.window_text()

                        if "IPI 0%" in ipi_value:
                            console.print(
                                f"Trabalhando com os itens, sucesso ao selecionar o valor do IPI ... \n"
                            )
                        else:
                            return RpaRetornoProcessoDTO(
                                sucesso=False,
                                retorno=f"Erro ao selecionar o IPI de unidade nos itens, IPI: {ipi_value}",
                                status=RpaHistoricoStatusEnum.Falha,
                            )

                        await worker_sleep(4)

                    try:
                        get_unidade = main_window.child_window(
                            class_name="TDBIComboBox", found_index=1
                        )
                        console.print(
                            f"VALOR ATUALMENTE SELECIONADO {get_unidade} ...\n"
                        )
                        if str(cod_split) in get_unidade.window_text():
                            console.print(
                                f"UNIDADE SELECIONADA CORRETAMENTE {get_unidade} ...\n"
                            )
                        else:
                            console.print(
                                f"SELECIONANDO UN-{cod_split} - F PARA UNIDADE ...\n"
                            )
                            get_unidade = main_window.child_window(
                                class_name="TDBIComboBox", found_index=1
                            )
                            combo_box_group = "UN-" + cod_split + " - F"
                            if str(cod_split) in get_unidade.window_text():
                                get_unidade.click_input()
                                set_combobox("||List", combo_box_group)
                                await worker_sleep(4)

                            # VERIFICANDO SE FOI SELECIONADO CORRETAMENTE
                            console.print(
                                f"VERIFICANDO SE FOI SELECIONADO UN-{cod_split} - F PARA UNIDADE ...\n"
                            )
                            get_unidade = main_window.child_window(
                                class_name="TDBIComboBox", found_index=1
                            )
                            if str(cod_split) in get_unidade.window_text():
                                combo_box_group = "UN-" + cod_split + " - F"
                                console.print(
                                    f"NÃO FOI SELECIONADO UN-{cod_split} - F PARA UNIDADE, TENTANDO SELECIONAR COMO CAIXA...\n"
                                )
                                num_adjusted = cod_split
                                if int(cod_split) < 9:
                                    num_adjusted = "0" + str(cod_split)
                                combo_box_group = "CAIXA C/" + num_adjusted
                                get_unidade.click_input()
                                set_combobox("||List", combo_box_group)
                                await worker_sleep(4)

                            get_unidade = main_window.child_window(
                                class_name="TDBIComboBox", found_index=1
                            )
                            if str(cod_split) in get_unidade.window_text():
                                return RpaRetornoProcessoDTO(
                                    sucesso=False,
                                    retorno=f"Erro ao selecionar o tipo de unidade nos itens, item: {n_item} {descricao}, não possui UN-{cod_split} - F OU CAIXA C/{cod_split} ",
                                    status=RpaHistoricoStatusEnum.Falha,
                                )

                            else:
                                console.print(
                                    f"SELECIONADO COMO CAIXA PARA UNIDADE...\n"
                                )

                    except Exception as e:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno=f"Erro ao selecionar o tipo de unidade nos itens: {e}",
                            status=RpaHistoricoStatusEnum.Falha,
                        )
                    
                    console.print(
                        f"Iten selecionado com sucesso, clicando em alterar ...\n"
                    )
                    await worker_sleep(2)

                    try:
                        btn_alterar = main_window.child_window(title="&Alterar")
                        btn_alterar.click()
                    except:
                        btn_alterar = main_window.child_window(title="Alterar")
                        btn_alterar.click()
                    await worker_sleep(3)

        except Exception as e:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao trabalhar nas alterações dos itens: {e}",
                status=RpaHistoricoStatusEnum.Falha,
            )

        await worker_sleep(2)
        # Inclui registro
        console.print(f"Incluindo registro...\n")
        try:
            ASSETS_PATH = "assets"
            inserir_registro = pyautogui.locateOnScreen(
                ASSETS_PATH + "\\entrada_notas\\IncluirRegistro.png", confidence=0.8
            )
            pyautogui.click(inserir_registro)
        except Exception as e:
            console.print(
                f"Não foi possivel incluir o registro utilizando reconhecimento de imagem, Error: {e}...\n tentando inserir via posição...\n"
            )
            await incluir_registro()

        await worker_sleep(3)
        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")

        await worker_sleep(2)

        # Verifica se a info 'Nota fiscal incluida' está na tela
        await worker_sleep(6)
        nf_imported = await check_nota_importada(nota.get("nfe"))
        if nf_imported.sucesso == True:
            await worker_sleep(3)
            console.print("\nVerifica se a nota ja foi lançada...")
            nf_chave_acesso = int(nota.get("nfe"))
            status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
            if status_nf_emsys.get("status") == "Lançada":
                console.print("\nNota lançada com sucesso, processo finalizado...", style="bold green")
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="Nota Lançada com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            else:
                console.print("Erro ao lançar nota", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Pop-up nota incluida encontrada, porém nota não encontrada no EMSys - Reprocessar",
                    status=RpaHistoricoStatusEnum.Falha,
                )
        else:
            console.print("Erro ao lançar nota", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao lançar nota, erro: {nf_imported.retorno}",
                status=RpaHistoricoStatusEnum.Falha,
            )

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}

    finally:
        # Deleta o xml
        await delete_xml(nota.get("nfe"))


import asyncio
import os
import subprocess
import time

import pyautogui
import pygetwindow._pygetwindow_win as gw
from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from pygetwindow._pygetwindow_win import Win32Window
from rich.console import Console

from worker_automate_hub.utils.logger import logger

console = Console()


def tirar_screenshot(nome_etapa: str):
    """
    Tira um screenshot da tela atual e salva em um arquivo com o nome
    especificado e a data/hora atual.

    :param nome_etapa: nome da etapa que est  sendo executada
    :type nome_etapa: str
    :return: caminho do arquivo de screenshot gerado
    :rtype: str
    """
    if nome_etapa is None:
        raise ValueError("Nome da etapa não pode ser nulo")

    caminho_screenshot = f"{nome_etapa}_{int(time.time())}.png"
    try:
        pyautogui.screenshot(caminho_screenshot)
        console.print(f"Screenshot tirada: {caminho_screenshot}")
        return caminho_screenshot

    except Exception as e:
        console.print(f"Erro ao tirar screenshot: {e}")
        return None


def deletar_screenshots(caminhos_screenshots: str):
    """
    Recebe uma lista de caminhos de arquivos de screenshots e deleta cada um deles.

    :param caminhos_screenshots: lista de caminhos de arquivos de screenshots
    :type caminhos_screenshots: list[str]
    """
    if caminhos_screenshots is None:
        raise ValueError("Lista de caminhos de screenshots não pode ser nula")

    for caminho in caminhos_screenshots:
        if caminho is None:
            raise ValueError("Caminho de screenshot não pode ser nulo")

        try:
            os.remove(caminho)
            console.print(f"Screenshot deletada: {caminho}")
        except OSError as e:
            console.print(f"Erro ao deletar screenshot {caminho}: {e}")


def fechar_janelas_rdp_sem_ip():
    """
    Fecha as janelas de conex o RDP sem IP.

    Caso não tenha nenhuma conexão RDP aberta, não fará  nada.
    """
    janelas_rdp = [
        win
        for win in gw.getAllTitles()
        if "Conexão de  Área de Trabalho Remota" in win
        or "Remote Desktop Connection" in win
    ]
    for titulo in janelas_rdp:
        if not any(char.isdigit() for char in titulo):
            console.print(f"Fechando pop-up de conexão sem IP: {titulo}")
            janela: Win32Window = gw.getWindowsWithTitle(titulo)[0]
            if janela is None:
                raise RuntimeError(f"Janela {titulo} não existe")
            try:
                janela.close()
            except Exception as e:
                logger.error(f"Erro ao fechar janela {titulo}: {e}")
            time.sleep(2)


def fechar_janela_existente(ip: str):
    """
    Fecha a janela existente com o IP informado.

    Args:
        ip (str): IP da janela a ser fechada.

    Raises:
        Exception: Erro ao tentar fechar a janela.
    """
    try:
        janelas_encontradas = gw.getAllTitles()
        for titulo in janelas_encontradas:
            if ip in titulo:
                janela: Win32Window = gw.getWindowsWithTitle(titulo)[0]
                if janela is None:
                    raise RuntimeError(f"Janela {titulo} não existe")
                console.print(f"Fechando janela existente: {titulo}")
                janela.activate()
                pyautogui.hotkey("alt", "f4")
                time.sleep(1)
                break
        else:
            console.print(f"Nenhuma janela encontrada com o IP: {ip}")

        fechar_janelas_rdp_sem_ip()

    except Exception as e:
        console.print(f"Erro ao tentar fechar a janela: {e}", style="bold red")


def restaurar_janelas_rdp():
    """
    Função para restaurar todas as janelas de Conexão de área de Trabalho Remota
    que estão minimizadas e mover elas para uma posicão na tela.
    """
    janelas_rdp = [
        win
        for win in gw.getAllTitles()
        if "Conexão de Área de Trabalho Remota" in win
        or "Remote Desktop Connection" in win
    ]

    offset_x = 0
    offset_y = 0
    step_x = 30
    step_y = 30

    for titulo in janelas_rdp:
        janela: Win32Window = gw.getWindowsWithTitle(titulo)[0]
        if janela is None:
            console.print(
                f"Erro ao restaurar janela {titulo}: janela não existe",
                style="bold red",
            )
            continue

        console.print(f"Processando janela: {titulo}")
        if janela.isMinimized:
            try:
                janela.restore()
            except Exception as e:
                console.print(
                    f"Erro ao restaurar janela {titulo}: {e}", style="bold red"
                )
            else:
                console.print(f"Janela restaurada: {titulo}")
        else:
            console.print(f"Janela já está aberta: {titulo}")

        try:
            janela.moveTo(offset_x, offset_y)
        except Exception as e:
            console.print(f"Erro ao mover janela {titulo}: {e}", style="bold red")
        else:
            offset_x += step_x
            offset_y += step_y

        try:
            janela.activate()
        except Exception as e:
            console.print(f"Erro ao ativar janela {titulo}: {e}", style="bold red")
        time.sleep(2)


def redimensionar_janela_rdp(largura: int, altura: int):
    """
    Redimensiona a janela da Conexão de Área de Trabalho Remota.

    :param largura: Largura da janela em pixels
    :type largura: int
    :param altura: Altura da janela em pixels
    :type altura: int
    """
    janelas_rdp = [
        win
        for win in gw.getAllTitles()
        if "Conexão de Área de Trabalho Remota" in win
        or "Remote Desktop Connection" in win
    ]

    if janelas_rdp:
        try:
            janela_rdp: Win32Window = gw.getWindowsWithTitle(janelas_rdp[0])[0]
            if janela_rdp is None:
                raise RuntimeError("Janela RDP não existe")
            janela_rdp.resizeTo(largura, altura)
            janela_rdp.moveTo(20, 20)
            console.print(f"Janela redimensionada para {largura}x{altura}.")
            janela_rdp.activate()
            janela_rdp.restore()
            time.sleep(1)
        except Exception as e:
            console.print(f"Erro ao redimensionar janela RDP: {e}", style="bold red")
    else:
        console.print("Não foi possível encontrar a janela RDP para redimensionar.")


async def conexao_rdp(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:

    caminhos_screenshots = []
    try:
        if not task or not task.configEntrada:
            raise ValueError("Task inválida ou sem configurações de entrada.")

        ip = task.configEntrada.get("ip", "")
        user = task.configEntrada.get("user", "")
        password = task.configEntrada.get("password", "")

        if not ip or not user or not password:
            raise ValueError(
                "Configurações de entrada inválidas. Verifique se o IP, Usuário e Senha estão preenchidos."
            )

        pyautogui.hotkey("win", "d")
        console.print("1 - Minimizando todas as telas...")
        await asyncio.sleep(2)

        fechar_janela_existente(ip)

        subprocess.Popen("mstsc")
        console.print("2 - Abrindo conexão de trabalho remota...")
        await asyncio.sleep(2)

        redimensionar_janela_rdp(500, 500)

        await asyncio.sleep(2)

        janelas_rdp = [
            win
            for win in gw.getAllTitles()
            if "Conexão de Área de Trabalho Remota" in win
            or "Remote Desktop Connection" in win
        ]
        if not janelas_rdp:
            raise RuntimeError("Nenhuma janela RDP foi encontrada.")

        janela_rdp: Win32Window = gw.getWindowsWithTitle(janelas_rdp[0])[0]
        if janela_rdp is None:
            raise RuntimeError("Janela RDP não existe.")

        janela_rdp.activate()
        janela_rdp.restore()
        await asyncio.sleep(1)

        caminhos_screenshots.append(tirar_screenshot("antes_de_inserir_ip"))
        console.print("3 - Inserindo o IP...")
        pyautogui.write(ip)
        await asyncio.sleep(10)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_ip"))
        pyautogui.press("enter")
        await asyncio.sleep(5)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_usuario"))
        await asyncio.sleep(5)

        console.print("5 - Inserindo a Senha...")
        pyautogui.write(password)
        pyautogui.press("enter")
        await asyncio.sleep(10)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_senha"))

        console.print("6 - Apertando left...")
        pyautogui.press("left")
        await asyncio.sleep(2)
        console.print("7 - Apertando Enter...")
        pyautogui.press("enter")
        await asyncio.sleep(20)
        caminhos_screenshots.append(tirar_screenshot("depois_do_certificado"))

        console.print("8 - Minimizando todas as telas no final...")
        pyautogui.hotkey("win", "d")
        await asyncio.sleep(2)
        caminhos_screenshots.append(tirar_screenshot("depois_de_minimizar_todas"))

        restaurar_janelas_rdp()
        caminhos_screenshots.append(tirar_screenshot("depois_de_restaurar_janelas"))

        deletar_screenshots(caminhos_screenshots)

        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Processo de conexão ao RDP executado com sucesso.",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    except Exception as ex:
        err_msg = f"Erro ao executar conexao_rdp: {ex}"
        logger.error(err_msg)
        console.print(err_msg, style="bold red")
        caminhos_screenshots.append(tirar_screenshot("erro"))
        deletar_screenshots(caminhos_screenshots)
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=err_msg,
            status=RpaHistoricoStatusEnum.Falha,
        )



import os
import subprocess
import sys
import threading
from pathlib import Path
from typing import Optional

import requests
import toml
from packaging import version
from rich.console import Console

from worker_automate_hub.utils.logger import logger

console = Console()


def update_version_in_toml(file_path: Path, new_version: str):
    """
    Atualiza a versão do worker no arquivo TOML.

    Args:
        file_path (Path): Caminho completo para o arquivo TOML.
        new_version (str): Nova versão para ser escrita no arquivo TOML.
    """
    try:
        # Abrir e carregar o arquivo TOML
        with open(file_path, "r") as file:
            config = toml.load(file)

        # Alterar a versão
        config["params"]["version"] = new_version

        # Salvar as alterações de volta no arquivo TOML
        with open(file_path, "w") as file:
            toml.dump(config, file)

        console.print(f"Versão atualizada para {new_version} no arquivo {file_path}")
    except Exception as e:
        console.print(f"Erro ao atualizar a versão: {e}")

def create_update_script(new_version: str):
    """
    Cria um arquivo de script .bat para atualizar a versão do worker com o pipx e reiniciar a aplicação.

    Args:
        new_version (str): Nova versão para ser escrita no arquivo TOML.

    Returns:
        Path: Caminho completo para o arquivo update.bat criado.
    """
    try:
        current_dir = Path.cwd()

        # Caminho do arquivo TOML
        toml_file_path = os.path.join(current_dir, 'settings.toml')        

        # Comando para atualizar o pacote via pipx
        update_command = 'pipx upgrade worker-automate-hub --force'

        # Comando para atualizar a versão no arquivo TOML
        update_toml_command = f'python -c "import toml; config=toml.load(\'{toml_file_path}\'); config[\'params\'][\'version\'] = \'{new_version}\'; toml.dump(config, open(\'{toml_file_path}\', \'w\'))"'

        # Comando para reiniciar a aplicação
        restart_command = f'start cmd /K "cd /D {current_dir} && worker-startup.bat"'

        # Caminho do arquivo de log
        log_file_path = os.path.join(current_dir, 'update.log')

        # Criando o conteúdo do script .bat
        script_content = f"""
        @echo off
        taskkill /IM worker.exe /F
        timeout /t 10 /nobreak
        {update_command}
        if %errorlevel% equ 0 (
            echo Update successful, updating version in TOML file...
            {update_toml_command}
            echo Version updated successfully, restarting the application...
            {restart_command}
        ) else (
            echo Update failed. > "{log_file_path}"
            echo Error code: %errorlevel% >> "{log_file_path}"
            echo See pipx output for more details. >> "{log_file_path}"
        )
        exit
        """

        # Caminho completo para o arquivo update.bat
        bat_file_path = os.path.join(current_dir, 'update.bat')

        # Escrevendo o conteúdo no arquivo update.bat
        with open(bat_file_path, 'w') as file:
            file.write(script_content.strip())

        log_msg = "Arquivo de update criado com sucesso!"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="bold green")
        
        return bat_file_path

    except Exception as e:
        err_msg = "Erro ao criar o arquivo de update: {e}"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")


def run_update_script(
    bat_file_path: str,
    stop_event: threading.Event
) -> None:
    """
    Executa o script .bat para atualizar o pacote.

    Args:
    - bat_file_path (str): Caminho completo para o arquivo .bat.
    - stop_event (threading.Event): Um objeto Event que é usado para
      sinalizar para as threads que devem parar.

    Returns:
    None
    """
    try:
        # Executando o script .bat
        subprocess.Popen(['start', 'cmd', '/c', bat_file_path], shell=True)
        
        log_msg = "update.bat executado com sucesso!"
        console.print(f"\n{log_msg}\n", style="bold green")
        logger.info(log_msg)        

        # Sinalizar para as threads que devem parar
        stop_event.set()

        # Encerrar o programa atual
        os.system('exit')
        sys.exit(0)        

    except subprocess.CalledProcessError as e:
        err_msg = f"Erro ao executar o update.bat: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(f"{err_msg}")

def get_installed_version(package_name: str) -> Optional[str]:
    
    """
    Retorna a versão atual do pacote instalada.

    Args:
    - package_name (str): Nome do pacote que se deseja obter a versão.

    Returns:
    - str: A versão atual do pacote ou None se o pacote não for encontrado.
    """
    try:
        result = subprocess.run(['pipx', 'list'], capture_output=True, text=True, check=True)
        lines = result.stdout.splitlines()
        for line in lines:
            if package_name in line:
                # Parse the version from the line, assuming the line format is 'package_name <version>'
                return line.split(' ')[5].replace(',', '')
    except subprocess.CalledProcessError as e:
        err_msg = f"Ocorreu um erro ao obter a versão do pacote instalada: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)
    return None

def check_for_update(stop_event: threading.Event):
    """
    Verifica se há uma nova versão do pacote disponível no PyPI.

    Se houver uma versão mais recente disponível, atualiza o pacote via pipx e
    reinicia a aplicação.

    Args:
        stop_event (threading.Event): Um objeto Event que é usado para
            sinalizar para as threads que devem parar.
    """
    try:
        package_name = "worker-automate-hub"
        current_version = get_installed_version(package_name)

        response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
        latest_version = response.json()
        console.print("last: "+latest_version["info"]["version"])
        console.print("current: "+current_version)
        if version.parse(latest_version["info"]["version"]) > version.parse(current_version):
            console.print(
                f"Uma nova versão [bold cyan]({latest_version["info"]["version"]})[/bold cyan] está disponível. Atualizando..."
            )
            bat_file_path = create_update_script(latest_version["info"]["version"])
            run_update_script(bat_file_path, stop_event)
        else:            
            current_dir = Path.cwd()

            # Caminho do arquivo TOML
            toml_file_path = os.path.join(current_dir, 'settings.toml')
            update_version_in_toml(toml_file_path, current_version)
            console.print("\nVocê está usando a versão mais atualizada.\n", style="green")
    except Exception as e:
        err_msg = f"Erro ao verificar novas versões do pacote: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)



from pydantic import BaseModel, Field


class RpaConfiguracao(BaseModel):
    uuidConfiguracao: str = Field(..., alias="uuidConfiguracao")
    labConfiguracao: str = Field(..., alias="labConfiguracao")
    conConfiguracao: dict = Field(None, alias="conConfiguracao")

    class Config:
        populate_by_name = True



import threading

import aiohttp
from aiohttp import ClientSession
import requests
from rich.console import Console

from worker_automate_hub.api.helpers.api_helpers import handle_api_response
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.models.dao.rpa_configuracao import RpaConfiguracao
from worker_automate_hub.models.dao.rpa_processo import RpaProcesso
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import get_new_task_info, get_system_info

console = Console()


async def get_new_task(stop_event: threading.Event) -> RpaProcessoEntradaDTO:
    env_config, _ = load_env_config()
    try: 
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        data = await get_new_task_info()

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{env_config["API_BASE_URL"]}/robo/new-job",
                data=data,
                headers=headers_basic,
            ) as response:
                res = await handle_api_response(response, stop_event)
                if res is not None:
                    return RpaProcessoEntradaDTO(**res.get('data'))
                else:
                    return None

    except Exception as e:
        err_msg = f"Erro ao obter nova tarefa: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None

async def burnQueue(id_fila: str):
    env_config, _ = load_env_config()
    try:       

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.delete(
                f"{env_config["API_BASE_URL"]}/fila/burn-queue/{id_fila}",
                headers=headers_basic,
            ) as response:
                if response.status == 200:
                    logger.info("Fila excluida com sucesso.")
                    console.print("\nFila excluida com sucesso.\n", style="bold green")    
                else:
                    logger.error(f"Erro ao excluir a fila: {response.content}") 
                    console.print(f"Erro ao excluir a fila: {response.content}", style="bold red")          

    except Exception as e:
        err_msg = f"Erro remover registro da fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
async def notify_is_alive(stop_event: threading.Event):
    env_config, _ = load_env_config()
    try:       

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        data = await get_system_info()

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.put(
                f"{env_config["API_BASE_URL"]}/robo/last-alive",
                data=data,
                headers=headers_basic,
            ) as response:
                return await handle_api_response(response, stop_event, last_alive=True)

    except Exception as e:
        err_msg = f"Erro ao informar is alive: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None

async def get_historico_by_processo_identificador(identificador, process_name):
    env_config, _ = load_env_config()
    
    if not identificador:
        return {"sucesso": False, "retorno": "Identificador do processo deve ser informado!"}
    elif not process_name:
        return {"sucesso": False, "retorno": "Nomedo do processo deve ser informado!"}
    
    headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}   
    
    # Corpo da requisição
    body = {
            "identificador": identificador,
            "nomProcesso": process_name,
            "pageSize": 1,
            "periodoBusca": 2,
        }
    
    try:
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{env_config['API_BASE_URL']}/historico/by-identificador",                
                headers=headers_basic,
                json=body,
            ) as response:
                if response.status != 200:
                    return {"sucesso": False, "retorno": f"Erro na requisição. Status code: {response.status}, {await response.text()}"}
                                
                # Processando a resposta JSON
                data = await response.json()
                
                if len(data) > 0:
                    return {"sucesso": True, "retorno": data}
                else:
                    return {"sucesso": False, "retorno": "Nenhum histórico encontrado para o processo"}
    except Exception as e:
        return {"sucesso": False, "retorno": f"Erro ao buscar o histórico: {str(e)}"}


async def get_processo(uuidProcesso: str) -> RpaProcesso:
    """
    Retorna o processo com base no uuid informado.

    Args:
        uuidProcesso (str): O uuid do processo a ser retornado.

    Raises:
        ValueError: Se o uuid do processo n o for informado.
        Exception: Se houver um erro ao obter o processo.

    Returns:
        RpaProcesso: O processo caso tenha sido encontrado.
    """
    env_config, _ = load_env_config()
    try:
        if not uuidProcesso:
            raise ValueError("O uuid do processo deve ser informado")
        
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}       
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/processo/{uuidProcesso}",                
                headers=headers_basic,
            ) as response:
                if response.status != 200:
                    error_content = await response.text()
                    raise Exception(f"Erro ao obter o processo: {error_content}")
                res = await response.json()
                if type(res["campos"]) == str and res["campos"] == "{}":
                    res["campos"] = {}
                return RpaProcesso(**res)

    except ValueError as e:
        logger.error(f"Erro ao obter o processo: {str(e)}")
        console.print(
            f"{e}\n",
            style="bold red",
        )
    except Exception as e:
        logger.error(f"Erro ao obter o processo: {str(e)}")
        console.print(
            f"{e}\n",
            style="bold red",
        )
        return None
async def get_workers():
    env_config, _ = load_env_config()
    try:
        

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/robo/workers",
                headers=headers_basic,
            ) as response:
                return await response.json()

    except Exception as e:
        err_msg = f"Erro ao obter a lista de workers: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None

async def get_config_by_name(name: str) -> RpaConfiguracao:
    """
    Obtem uma configuração pelo nome.

    Args:
        name (str): Nome da configuração a ser obtida.

    Returns:
        RpaConfiguracao: A configuração obtida.

    Raises:
        ValueError: Se não houver configuração do ambiente carregada.
        ValueError: Se não houver chave de autentica o na configuração do ambiente.
        ValueError: Se não houver URL da API na configuração do ambiente.
        Exception: Se houver um erro ao obter a configuração.
    """
    env_config, _ = load_env_config()
    if env_config is None:
        raise ValueError("Configuração do ambiente não carregada")
    if "API_AUTHORIZATION" not in env_config:
        raise ValueError("Chave de autenticação não encontrada na configuração do ambiente")
    if "API_BASE_URL" not in env_config:
        raise ValueError("URL da API não encontrada na configuração do ambiente")
    
    try:
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/configuracao/{name}",
                headers=headers_basic,
            ) as response:
                if response.status != 200:
                    raise Exception(f"Erro ao obter a configuração: {response.content}")
                data = await response.json()
                return RpaConfiguracao(**data)

    except Exception as e:
        err_msg = f"Erro ao obter a configuração: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
def sync_get_config_by_name(name: str) -> RpaConfiguracao:
    """
    Obtém a configuração do ambiente pelo nome.

    Args:
        name (str): Nome da configuração a ser obtida.

    Returns:
        RpaConfiguracao: A configuração obtida.

    Raises:
        ValueError: Se não houver configuração do ambiente carregada.
        ValueError: Se não houver chave de autenticação na configuração do ambiente.
        ValueError: Se não houver URL da API na configuração do ambiente.
        Exception: Se houver um erro ao obter a configuração.
    """
    env_config, _ = load_env_config()
    if env_config is None:
        raise ValueError("Configuração do ambiente não carregada")
    if "API_AUTHORIZATION" not in env_config:
        raise ValueError("Chave de autenticação não encontrada na configuração do ambiente")
    if "API_BASE_URL" not in env_config:
        raise ValueError("URL da API não encontrada na configuração do ambiente")
    
    try:
        headers_basic = {"Authorization": f"Basic {env_config['API_AUTHORIZATION']}"}

        response = requests.get(
            f"{env_config['API_BASE_URL']}/configuracao/{name}",
            headers=headers_basic,
            verify=False  # Desativa a verificação SSL
        )

        response.raise_for_status()
        
        data = response.json()
        return RpaConfiguracao(**data)

    except requests.RequestException as e:
        err_msg = f"Erro ao obter a configuração: {e}"
        logger.error(err_msg)
        console.print(err_msg, style="red")
        return None
    
async def send_gchat_message(message: str) -> None:
    """
    Envia uma mensagem para o Google Chat.

    Args:
        message (str): Mensagem a ser enviada.

    Returns:
        dict: O retorno da API do Google Chat.

    Raises:
        ValueError: Se não houver configuração do ambiente carregada.
        ValueError: Se não houver chave de autenticação na configuração do ambiente.
        ValueError: Se não houver URL da API na configuração do ambiente.
        Exception: Se houver um erro ao enviar a mensagem.
    """
    env_config, _ = load_env_config()
    if env_config is None:
        raise ValueError("Configurão do ambiente não carregada")
    if "API_AUTHORIZATION" not in env_config:
        raise ValueError("Chave de autenticação não encontrada na configuração do ambiente")
    if "API_BASE_URL" not in env_config:
        raise ValueError("URL da API não encontrada na configuração do ambiente")

    try:
        headers_basic = {"Authorization": f"Basic {env_config['API_AUTHORIZATION']}"}

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{env_config['API_BASE_URL']}/google-chat",
                data={"message": message},
                headers=headers_basic,
            ) as response:
                if response.status != 200:
                    raise Exception(f"Erro ao enviar mensagem ao Google Chat: {response.content}")
                

    except ValueError as e:
        logger.error(f"Erro ao enviar mensagem ao Google Chat: {e}")
        console.print(
            f"{e}\n",
            style="bold red",
        )
        return None
    except Exception as e:
        logger.error(f"Erro ao enviar mensagem ao Google Chat: {e}")
        console.print(
            f"{e}\n",
            style="bold red",
        )
        return None
    
async def unlock_queue(id: str):
    env_config, _ = load_env_config()
    try:      
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}       
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/fila/unlock-queue/{id}",                
                headers=headers_basic,
            ) as response:
                return await response.text()

    except Exception as e:
        err_msg = f"Erro ao desbloquear a fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None


def read_secret(path: str, vault_token: str):
    

    url = f"https://aspirina.simtech.solutions/{path}"
    headers = {"X-Vault-Token": vault_token, "Content-Type": "application/json"}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        result = response.json()
        return result["data"]["data"]
    elif response.status_code == 403:
        err_msg = "403 - Token inválido!"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")
    else:
        response.raise_for_status()


def load_environments(env: str, vault_token: str):

    environments = {}   
    credentials = {}

    environments[env] = read_secret(path=f"v1/{env}-sim/data/worker-automate-hub/env", vault_token=vault_token)
    credentials[env] = read_secret(path=f"v1/{env}-sim/data/worker-automate-hub/credentials.json", vault_token=vault_token)

    return environments[env], credentials[env]


async def get_index_modelo_emsys(filial: str, descricao_documento: str):
    """
    Procura o index de um modelo de documento fiscal no EMSYS.

    Args:
    filial (str): Código da filial.
    descricao_documento (str): Descrição do documento fiscal.

    Returns:
    dict: Contendo o index do modelo de documento fiscal.

    Raises:
    Exception: Se houver um erro ao comunicar com o endpoint do Simplifica.
    """
    env_config, _ = load_env_config()
    
    body = {
        "codigoEmpresa": filial,
        "descricaoDocumento": descricao_documento
        }
    headers_basic = {"Authorization": f"Basic {env_config['API_AUTHORIZATION']}"} 

    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            async with session.post(f"{env_config['API_BASE_URL']}/emsys/buscar-index-documento-fiscal", data=body, 
                                    headers=headers_basic) as response:
                if response.status != 200:
                    raise Exception(f"Erro ao comunicar com endpoint do Simplifica: {response.text}")
                data = await response.json()
                if not data or not isinstance(data, dict):
                    raise Exception(f"Erro ao comunicar com endpoint do Simplifica: {data}")
                log_msg = f"\nSucesso ao procurar {data}.\n"
                console.print(
                    log_msg,
                    style="bold green",
                )
                logger.info(log_msg)
                return data

    except Exception as e:
        err_msg = f"Erro ao comunicar com endpoint do Simplifica: {e}"
        console.print(f"\n{err_msg}\n", style="bold green")
        logger.info(err_msg)


async def get_status_nf_emsys(chave: int):
    """
    Procura o status de nota fiscal no  EMSYS.

    Args:
    chave (int): Chave de acesso a NF.

    Returns:
    dict: Contendo o chave de acesso e status de processamento da nota.

    Raises:
    Exception: Se houver um erro ao comunicar com o endpoint do Simplifica.
    """
    env_config, _ = load_env_config()

    url = f"{env_config['API_BASE_URL']}/emsys/consulta-status-nota?chaveNfe={chave}"
    
    headers_basic = {"Authorization": f"Basic {env_config['API_AUTHORIZATION']}"} 

    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            async with session.get(url, headers=headers_basic) as response:
                if response.status != 200:
                    raise Exception(f"Erro ao comunicar com endpoint do Simplifica: {response.text}")
                data = await response.json()
                if not data or not isinstance(data, dict):
                    raise Exception(f"Erro ao comunicar com endpoint do Simplifica: {data}")
                log_msg = f"\nSucesso ao procurar {data}.\n"
                console.print(
                    log_msg,
                    style="bold green",
                )
                logger.info(log_msg)
                return data

    except Exception as e:
        err_msg = f"Erro ao comunicar com endpoint do Simplifica: {e}"
        console.print(f"\n{err_msg}\n", style="bold green")
        logger.info(err_msg)

# Função para enviar arquivo de imagem a api
async def send_file(uuidRelacao: str, desArquivo: str, tipo: str, file: bytes, file_extension: str = "jpg") -> None:
    """
    Função assíncrona para enviar um arquivo de imagem para uma API.

    Args:
        uuidRelacao (str): UUID da relação associada ao arquivo.
        desArquivo (str): Descrição do arquivo.
        tipo (str): Tipo de arquivo.
        file (bytes): Conteúdo binário do arquivo.
    """
    try:
        # Carrega as configurações de ambiente
        env_config, _ = load_env_config()

        if file_extension == "txt":
            filename = "file.txt"
            content_type = "text/plain"
        else:
            filename = "file.jpg"
            content_type = "image/jpeg"

        # Criação do corpo da requisição multipart
        body = aiohttp.FormData()
        body.add_field('uuidRelacao', uuidRelacao)
        body.add_field('desArquivo', desArquivo)
        body.add_field('tipo', tipo)
        body.add_field('file', file, filename=filename, content_type=content_type)
        #body.add_field('file', file, filename="file.jpg", content_type="image/jpeg")

        headers_basic = {
            "Authorization": f"Basic {env_config['API_AUTHORIZATION']}"
        }

        # Enviando requisição para a API
        async with ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
            async with session.post(f"{env_config['API_BASE_URL']}/arquivo/send-file", data=body, headers=headers_basic) as response:
                response.raise_for_status()  # Levanta exceção se o status não for 2xx
                log_msg = f"\nSucesso ao enviar arquivo: {uuidRelacao}.\n"
                console.print(log_msg, style="bold green")
                logger.info(log_msg)

    except aiohttp.ClientResponseError as e:
        err_msg = f"Erro na resposta da API: {e.status} - {e.message}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)

    except Exception as e:
        err_msg = f"Erro ao enviar arquivo: {str(e)}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)


import asyncio
import warnings
from datetime import datetime, timedelta

import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.utils.logger import logger
from pywinauto_recorder import set_combobox
from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.util import (
    kill_process, 
    login_emsys, 
    type_text_into_field, 
    worker_sleep,
    set_variable,
    )

ASSETS_BASE_PATH = 'assets/fidc/'
console = Console()



async def remessa_cobranca_cnab240(task):
    '''
       Processo FIDC - Remessa de Cobrança CNAB240
    '''
    try:
        #Setando tempo de timeout
        multiplicador_timeout = int(float(task["sistemas"][0]["timeout"]))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        #Pega Config para logar no Emsys
        config = await get_config_by_name("login_emsys")
        #folders_paths = await get_config_by_name("Folders_Fidc")
        # console.print(task)
        # #Abre um novo emsys
        # await kill_process("EMSys")
        # app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        # warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        # console.print("\nEMSys iniciando...", style="bold green")
        # return_login = await login_emsys(config['conConfiguracao'], app, task)

        # if return_login['sucesso'] == True:
        #     type_text_into_field('Remessa de Cobrança', app['TFrmMenuPrincipal']['Edit'], True, '50')
        #     pyautogui.press('enter')
        #     await worker_sleep(1)
        #     pyautogui.press('enter')
        #     console.print(f"\nPesquisa: 'Impressao de Boletos' realizada com sucesso", style="bold green")
        # else:
        #     logger.info(f"\nError Message: {return_login["retorno"]}")
        #     console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
        #     return return_login
        
        # await worker_sleep(10)
        #Identificando jenela principal
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]

        #Digitando Cobrança
        # cobranca = main_window_arquivo_cobranca.child_window(class_name="TDBIEditCode", found_index=2)
        # console.print("Selecionando Cobrança", style='bold green')
        # cobranca.type_keys("4")
        # pyautogui.hotkey("tab")
        # await worker_sleep(5)
        # pyautogui.press("down", presses=3, interval=0.5)
        # pyautogui.hotkey("enter")
        
        #TODO passo 8 da IT


        # #Seleciona Banco
        # pyautogui.click(800, 395)
        # pyautogui.press("down", presses=2)
        # pyautogui.hotkey("enter")
        
        # Data atual
        data_atual = datetime.now()
        # Data(8 dias atrás)
        start_date = data_atual - timedelta(days=8)
        # Data(1 dia atrás)
        end_date = data_atual - timedelta(days=1)

        #Data de emissão
        pyautogui.click(690, 485)
        pyautogui.write(start_date.strftime("%d%m%Y"))
        pyautogui.click(780, 485)
        pyautogui.write(end_date.strftime("%d%m%Y"))

        print("")
    
    except Exception as ex:
        observacao = f"Erro Processo Remessa de Cobranca CNAB240: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}
    finally:
       ... # await kill_process("EMSys")


    

    



if __name__ == "__main__":
    task_fake = {
        "configEntrada":{
            "filialEmpresaOrigem": "1"
        },
        "sistemas": [
            {
            "sistema": "EMSys",
            "timeout": "1.0"
            },
            {
            "sistema": "AutoSystem",
            "timeout": "1.0"
            }
        ]
    }
    asyncio.run(remessa_cobranca_cnab240(task_fake))


import json
import tempfile

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow


class GetCredsGworkspace:
    def __init__(self, token_dict=None, credentials_dict=None, scopes=None):
        self.token_dict = token_dict
        self.credentials_dict = credentials_dict
        self.scopes = scopes or ["https://www.googleapis.com/auth/spreadsheets",
                                "https://www.googleapis.com/auth/drive",
                                "https://mail.google.com/",
                                ]

    def get_creds_gworkspace(self):
        creds = None
        try:
            if self.token_dict:
                with tempfile.NamedTemporaryFile(mode='w+', delete=False) as token_file:
                    json.dump(self.token_dict, token_file)
                    token_file.flush()
                    token_path = token_file.name

                    creds = Credentials.from_authorized_user_file(token_path, self.scopes)
            
            # Caso o arquivo de token não exista ou não seja válido
            if not creds or not creds.valid:
                if creds and creds.expired and creds.refresh_token:
                    creds.refresh(Request())
                else:
                    # Usar arquivo temporário para as credenciais
                    if self.credentials_dict:
                        with tempfile.NamedTemporaryFile(mode='w+', delete=False) as creds_file:
                            json.dump(self.credentials_dict, creds_file)
                            creds_file.flush()
                            creds_path = creds_file.name

                        flow = InstalledAppFlow.from_client_secrets_file(creds_path, self.scopes)
                        creds = flow.run_local_server(port=0)

                    # Escreve o novo token no arquivo temporário
                    with tempfile.NamedTemporaryFile(mode='w+', delete=False) as token_file:
                        token_file.write(creds.to_json())

        except Exception as e:
            return None

        return creds


import asyncio
import random

import pandas as pd
import pyautogui
from rich.progress import (
    BarColumn,
    Progress,
    SpinnerColumn,
    TextColumn,
    TimeRemainingColumn,
)

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger

# Variável global para armazenar o status atual da tarefa
current_task_status = {
    "step": 0,
    "total_steps": 0,
    "description": "Sem tarefa",
}


async def update_task_status(step, total_steps, description):
    global current_task_status
    current_task_status = {
        "step": step,
        "total_steps": total_steps,
        "description": description,
    }


async def exemplo_processo(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        num_actions = 10
        actions = []

        await update_task_status(0, num_actions, "Iniciando")

        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
            TimeRemainingColumn(),
        ) as progress:
            task = progress.add_task("Iniciando...", total=num_actions)

            for i in range(num_actions):
                screen_width, screen_height = pyautogui.size()
                x = random.randint(0, screen_width - 1)
                y = random.randint(0, screen_height - 1)

                action_type = random.choice(["move", "click", "write"])
                if action_type == "move":
                    pyautogui.moveTo(x, y, duration=random.uniform(0.1, 1.0))
                elif action_type == "click":
                    pyautogui.click(x, y)
                elif action_type == "write":
                    pyautogui.click(x, y)
                    pyautogui.write("Hello", interval=0.1)

                # Atualizando a variável global de status da tarefa
                await update_task_status(
                    i + 1,
                    num_actions,
                    f"Executando ação {i + 1}/{num_actions}: {action_type}",
                )

                # Atualizando a barra de progresso
                progress.update(
                    task,
                    advance=1,
                    description=f"Executando ação {i + 1}/{num_actions}: {action_type}",
                )

                await asyncio.sleep(12)

            filename = "random_actions.xlsx"
            df = pd.DataFrame(actions)
            df.to_excel(filename, index=False)

        logger.info("Processo executado com sucesso")
        print("Processo executado com sucesso")
        await update_task_status(0, 0, "Sem tarefa")
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Task executed successfully and data saved to Excel.",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    except Exception as e:
        logger.error(f"Erro ao executar o processo: {e}")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"An error occurred: {e}",
            status=RpaHistoricoStatusEnum.Falha,
        )
